@@ -38,8 +38,6 @@ from edgar.core import (edgar_mode,
                         NORMAL,
                         get_identity,
                         set_identity,
-                        use_local_storage,
-                        download_edgar_data,
                         listify)
 from edgar.fundreports import FundReport, NPORT_FORMS
 from edgar.funds import Fund, FundSeries, get_fund, FundClass
@@ -47,9 +45,12 @@ from edgar.thirteenf import ThirteenF, THIRTEENF_FORMS
 from edgar.files.html import Document
 from edgar.xbrl import XBRLData
 from edgar.financials import Financials, MultiFinancials
+from edgar.storage import use_local_storage, is_using_local_storage, download_edgar_data, download_filings
 
 # Another name for get_current_filings
 get_latest_filings = get_current_filings
+latest_filings = get_current_filings
+current_filings = get_current_filings
 
 # Fund portfolio report filings
 get_fund_portfolio_filings = partial(get_filings, form=NPORT_FORMS)
@@ -117,13 +118,15 @@ def obj(sec_filing: Filing) -> Optional[object]:
     :param sec_filing: The filing
     :return:
     """
-    from edgar.company_reports import TenK, TenQ, TwentyF, EightK
+    from edgar.company_reports import TenK, TenQ, TwentyF, EightK, CurrentReport
     from edgar.effect import Effect
     from edgar.offerings import FormC, FormD
     from edgar.ownership import Ownership, Form3, Form4, Form5
     from edgar.form144 import Form144
     from edgar.muniadvisors import MunicipalAdvisorForm
 
+    if matches_form(sec_filing, "6-K"):
+        return CurrentReport(sec_filing)
     if matches_form(sec_filing, "8-K"):
         return EightK(sec_filing)
     elif matches_form(sec_filing, "10-Q"):
