@@ -1,10 +1,10 @@
 import itertools
+import json
 import pickle
 import re
 import webbrowser
 from contextlib import nullcontext
 from dataclasses import dataclass
-from datetime import date
 from datetime import datetime
 from functools import lru_cache, cached_property
 from io import BytesIO
@@ -14,13 +14,11 @@ from typing import Tuple, List, Dict, Union, Optional, Any, cast
 
 import httpx
 import numpy as np
-import orjson as json
 import pandas as pd
 import pyarrow as pa
 import pyarrow.compute as pc
 import pyarrow.csv as pa_csv
 import pyarrow.parquet as pq
-import pytz
 from bs4 import BeautifulSoup
 from fastcore.parallel import parallel
 from rich import box
@@ -38,28 +36,40 @@ from edgar.core import (log, display_size, sec_edgar,
                         filter_by_date,
                         filter_by_form,
                         filter_by_cik,
+                        filter_by_exchange,
                         filter_by_ticker,
                         filter_by_accession_number,
                         listify,
                         cache_except_none,
                         is_start_of_quarter,
                         has_html_content,
-                        InvalidDateException, IntString, DataPager)
+                        InvalidDateException,
+                        IntString,
+                        current_year_and_quarter,
+                        Years,
+                        Quarters,
+                        YearAndQuarter,
+                        YearAndQuarters,
+                        quarters_in_year,
+                        filing_date_to_year_quarters,
+                        DataPager,
+                        PagingState)
 from edgar.files.html import Document
 from edgar.files.html_documents import get_clean_html
 from edgar.files.htmltools import html_sections
 from edgar.files.markdown import to_markdown
-from edgar.filingheader import FilingHeader
 from edgar.headers import FilingDirectory, IndexHeaders
 from edgar.httprequests import download_file, download_text, download_text_between_tags
 from edgar.httprequests import get_with_retry
 from edgar.reference import describe_form
-from edgar.richtools import df_to_rich_table, repr_rich, rich_to_text, print_rich
+from edgar.reference.tickers import Exchange
+from edgar.reference.tickers import find_ticker
+from edgar.richtools import repr_rich, print_rich, rich_to_text
 from edgar.search import BM25Search, RegexSearch
+from edgar.sgml import FilingSGML, Reports, Statements, FilingHeader
+from edgar.storage import local_filing_path, is_using_local_storage
 from edgar.xbrl import XBRLData, XBRLInstance, get_xbrl_object
 from edgar.xmltools import child_text
-from edgar.sgml import FilingSgml
-from edgar.reference.tickers import find_ticker
 
 """ Contain functionality for working with SEC filing indexes and filings
 
@@ -74,7 +84,7 @@ __all__ = [
     'Filings',
     'get_filings',
     'FilingHeader',
-    'FilingsState',
+    'PagingState',
     'Attachment',
     'Attachments',
     'FilingHomepage',
@@ -99,31 +109,12 @@ xbrl_index = "xbrl"
 company_index = "company"
 
 max_concurrent_http_connections = 10
-quarters_in_year: List[int] = list(range(1, 5))
-
-YearAndQuarter = Tuple[int, int]
-YearAndQuarters = List[YearAndQuarter]
-Years = Union[int, List[int], range]
-Quarters = Union[int, List[int], range]
 
 accession_number_re = re.compile(r"\d{10}-\d{2}-\d{6}$")
 
 xbrl_document_types = ['XBRL INSTANCE DOCUMENT', 'XBRL INSTANCE FILE', 'EXTRACTED XBRL INSTANCE DOCUMENT']
 
 
-def current_year_and_quarter() -> Tuple[int, int]:
-    # Define the Eastern timezone
-    eastern = pytz.timezone('America/New_York')
-
-    # Get the current time in Eastern timezone
-    now_eastern = datetime.now(eastern)
-
-    # Calculate the current year and quarter
-    current_year, current_quarter = now_eastern.year, (now_eastern.month - 1) // 3 + 1
-
-    return current_year, current_quarter
-
-
 def is_valid_filing_date(filing_date: str) -> bool:
     if ":" in filing_date:
         # Check for only one colon
@@ -176,8 +167,8 @@ def available_quarters() -> YearAndQuarters:
     return start_quarters + in_between_quarters + end_quarters
 
 
-def expand_quarters(year: Years,
-                    quarter: Optional[Quarters] = None) -> YearAndQuarters:
+def expand_quarters(year: Union[int, List[int]],
+                    quarter: Optional[Union[int, List[int]]] = None) -> YearAndQuarters:
     """
     Expand the list of years and a list of quarters to a full list of tuples covering the full range
     :param year: The year or years
@@ -192,43 +183,6 @@ def expand_quarters(year: Years,
             ]
 
 
-def filing_date_to_year_quarters(filing_date: str) -> List[Tuple[int, int]]:
-    if ":" in filing_date:
-        start_date, end_date = filing_date.split(":")
-
-        if not start_date:
-            start_date = "1994-06-01"
-
-        if not end_date:
-            end_date = date.today().strftime("%Y-%m-%d")
-
-        start_year, start_month, _ = map(int, start_date.split("-"))
-        end_year, end_month, _ = map(int, end_date.split("-"))
-
-        start_quarter = (start_month - 1) // 3 + 1
-        end_quarter = (end_month - 1) // 3 + 1
-
-        result = []
-        for year in range(start_year, end_year + 1):
-            if year == start_year and year == end_year:
-                quarters = range(start_quarter, end_quarter + 1)
-            elif year == start_year:
-                quarters = range(start_quarter, 5)
-            elif year == end_year:
-                quarters = range(1, end_quarter + 1)
-            else:
-                quarters = range(1, 5)
-
-            for quarter in quarters:
-                result.append((year, quarter))
-
-        return result
-    else:
-        year, month, _ = map(int, filing_date.split("-"))
-        quarter = (month - 1) // 3 + 1
-        return [(year, quarter)]
-
-
 class FileSpecs:
     """
     A specification for a fixed width file
@@ -314,7 +268,7 @@ def read_fixed_width_index(index_text: str,
     )
 
 
-def read_index_file(index_text: str, columns:List[str] = FORM_INDEX_COLUMNS) -> pa.Table:
+def read_index_file(index_text: str, columns: List[str] = FORM_INDEX_COLUMNS) -> pa.Table:
     """
     Read the index text using multiple spaces as delimiter
     """
@@ -333,7 +287,7 @@ def read_index_file(index_text: str, columns:List[str] = FORM_INDEX_COLUMNS) ->
     rows = [line.split() for line in data_lines if line.strip()]
 
     # Convert to arrays
-    forms = pa.array([line[:12].strip() for line in data_lines]) # The form might contain spaces like '1-A POS'
+    forms = pa.array([line[:12].strip() for line in data_lines])  # The form might contain spaces like '1-A POS'
 
     # Company names may have single spaces within them
     companies = pa.array([' '.join(row[1:-3]) for row in rows])
@@ -353,10 +307,12 @@ def read_index_file(index_text: str, columns:List[str] = FORM_INDEX_COLUMNS) ->
         names=columns
     )
 
+
 def read_form_index_file(index_text: str) -> pa.Table:
     """Read the form index file"""
     return read_index_file(index_text, columns=FORM_INDEX_COLUMNS)
 
+
 def read_company_index_file(index_text: str) -> pa.Table:
     """Read the company index file"""
     return read_index_file(index_text, columns=COMPANY_INDEX_COLUMNS)
@@ -447,7 +403,7 @@ def get_filings_for_quarters(year_and_quarters: YearAndQuarters,
     Get the filings for the quarters
     :param year_and_quarters:
     :param index: The index to use - "form", "company", or "xbrl"
-    :return:
+    :return: The filings as a pyarrow table
     """
 
     if len(year_and_quarters) == 1:
@@ -466,12 +422,6 @@ def get_filings_for_quarters(year_and_quarters: YearAndQuarters,
     return final_index_table
 
 
-@dataclass
-class FilingsState:
-    page_start: int
-    num_filings: int
-
-
 class Filings:
     """
     A container for filings
@@ -479,11 +429,11 @@ class Filings:
 
     def __init__(self,
                  filing_index: pa.Table,
-                 original_state: Optional[FilingsState] = None):
+                 original_state: Optional[PagingState] = None):
         self.data: pa.Table = filing_index
         self.data_pager = DataPager(self.data)
         # This keeps track of where the index should start in case this is just a page in the Filings
-        self._original_state = original_state or FilingsState(0, len(self.data))
+        self._original_state = original_state or PagingState(0, len(self.data))
 
     def to_pandas(self, *columns) -> pd.DataFrame:
         """Return the filing index as a python dataframe"""
@@ -534,12 +484,13 @@ class Filings:
             return filings[0]
         return filings
 
-    def filter(self,
+    def filter(self, *,
                form: Optional[Union[str, List[IntString]]] = None,
                amendments: bool = None,
                filing_date: Optional[str] = None,
                date: Optional[str] = None,
                cik: Union[IntString, List[IntString]] = None,
+               exchange: Union[str, List[str], Exchange, List[Exchange]] = None,
                ticker: Union[str, List[str]] = None,
                accession_number: Union[str, List[str]] = None) -> Optional['Filings']:
         """
@@ -566,6 +517,7 @@ class Filings:
         :param filing_date: The filing date
         :param date: An alias for the filing date
         :param cik: The CIK or list of CIKs to filter by
+        :param exchange: The exchange or list of exchanges to filter by
         :param ticker: The ticker or list of tickers to filter by
         :param accession_number: The accession number or list of accession numbers to filter by
         :return: The filtered filings
@@ -597,6 +549,10 @@ class Filings:
         if cik:
             filing_index = filter_by_cik(filing_index, cik)
 
+        # Filter by exchange
+        if exchange:
+            filing_index = filter_by_exchange(filing_index, exchange)
+
         if ticker:
             filing_index = filter_by_ticker(filing_index, ticker)
 
@@ -649,7 +605,7 @@ class Filings:
             log.warning("End of data .. use prev() \u2190 ")
             return None
         start_index, _ = self.data_pager._current_range
-        filings_state = FilingsState(page_start=start_index, num_filings=len(self))
+        filings_state = PagingState(page_start=start_index, num_records=len(self))
         return Filings(data_page, original_state=filings_state)
 
     def previous(self):
@@ -662,7 +618,7 @@ class Filings:
             log.warning(" No previous data .. use next() \u2192 ")
             return None
         start_index, _ = self.data_pager._current_range
-        filings_state = FilingsState(page_start=start_index, num_filings=len(self))
+        filings_state = PagingState(page_start=start_index, num_records=len(self))
         return Filings(data_page, original_state=filings_state)
 
     def prev(self):
@@ -736,7 +692,7 @@ class Filings:
     @property
     def summary(self):
         return (f"Showing {self.data_pager.page_size} of "
-                f"{self._original_state.num_filings:,} filings")
+                f"{self._original_state.num_records:,} filings")
 
     def _page_index(self) -> range:
         """Create the range index to set on the page dataframe depending on where in the data we are
@@ -795,7 +751,7 @@ class Filings:
         elements = [table]
 
         if self.data_pager.total_pages > 1:
-            total_filings = self._original_state.num_filings
+            total_filings = self._original_state.num_records
             current_count = len(current_page)
             start_num = start_idx + 1
             end_num = start_idx + current_count
@@ -844,8 +800,8 @@ def sort_filings_by_priority(filing_table: pa.Table,
         PyArrow table sorted by date and form priority
     """
     if priority_forms is None:
-        priority_forms = ['10-Q', '10-Q/A', '10-K', '10-K/A',  '8-K', '8-K/A',
-                          '6-K', '6-K/A', '13F-HR', '144',  '4', 'D', 'SC 13D', 'SC 13G']
+        priority_forms = ['10-Q', '10-Q/A', '10-K', '10-K/A', '8-K', '8-K/A',
+                          '6-K', '6-K/A', '13F-HR', '144', '4', 'D', 'SC 13D', 'SC 13G']
 
     # Create form priority values
     forms_array = filing_table['form']
@@ -955,7 +911,8 @@ def get_filings(year: Optional[Years] = None,
             # Ensure at least some data is returned
             previous_quarter = [get_previous_quarter(year, quarter)]
             filing_index = get_filings_for_quarters(previous_quarter, index=index)
-            return Filings(filing_index)
+            sorted_filing_index = sort_filings_by_priority(filings.data, priority_forms)
+            return Filings(sorted_filing_index)
         return None
 
     # Sort the filings using the separate sort function
@@ -1150,16 +1107,74 @@ class CurrentFilings(Filings):
         return None
 
     def __rich__(self):
-        page: pd.DataFrame = self.to_pandas()
+
+        # Create table with appropriate columns and styling
+        table = Table(
+            show_header=True,
+            header_style="bold",
+            show_edge=True,
+            expand=False,
+            padding=(0, 1),
+            box=box.SIMPLE,
+            row_styles=["", "bold"]
+        )
+
+        # Add columns with specific styling and alignment
+        table.add_column("#", style="dim", justify="right")
+        table.add_column("Form", width=7)
+        table.add_column("CIK", style="dim", width=10, justify="right")
+        table.add_column("Ticker", width=6, style="yellow")
+        table.add_column("Company", style="bold green", width=38, no_wrap=True)
+        table.add_column("Filing Date", width=11)
+        table.add_column("Accession Number", style="dim", width=20)
+
+        # Get current page from data pager
+        current_page = self.data.to_pandas()
 
         # compute the index from the start and page_size and set it as the index of the page
-        page.index = range(self._start - 1, self._start - 1 + len(page))
+        current_page.index = range(self._start - 1, self._start - 1 + len(current_page))
 
+        # Iterate through rows in current page
+        for t in current_page.itertuples():
+            cik = t.cik
+            ticker = find_ticker(cik)
+
+            row = [
+                str(t.Index),
+                t.form,
+                str(cik),
+                ticker,
+                t.company,
+                str(t.filing_date),
+                t.accession_number
+            ]
+            table.add_row(*row)
+
+        # Show paging information only if there are multiple pages
+        elements = [table]
+
+        page_info = Text.assemble(
+            ("Showing ", "dim"),
+            (f"{current_page.index.min():,}", "bold red"),
+            (" to ", "dim"),
+            (f"{current_page.index.max():,}", "bold red"),
+            (" most recent filings.", "dim"),
+            (" Page using ", "dim"),
+            ("← prev()", "bold gray54"),
+            (" and ", "dim"),
+            ("next() →", "bold gray54")
+        )
+
+        elements.extend([Text("\n"), page_info])
+
+        # Get the subtitle
+        start_date, end_date = self.date_range
+        subtitle = "Most recent filings from the SEC"
         return Panel(
-            Group(
-                df_to_rich_table(page),
-                Text(f"Filings {page.index.min()} to {page.index.max()}")
-            ), title=f"Current Filings on {self.start_date}"
+            Group(*elements),
+            title="SEC Filings",
+            subtitle=subtitle,
+            border_style="bold grey54"
         )
 
 
@@ -1239,6 +1254,10 @@ class Filing:
         """
         :return: The primary display document on the filing, generally HTML but can be XHTML
         """
+        document = self.sgml().attachments.primary_html_document
+        # If the document is not in the SGML then we have to go to the homepage
+        if document:
+            return document
         return self.homepage.primary_html_document
 
     @property
@@ -1246,29 +1265,55 @@ class Filing:
         """
         :return: a list of the primary documents on the filing, generally HTML or XHTML and optionally XML
         """
-        return self.homepage.primary_documents
+        documents = self.sgml().attachments.primary_documents
+        if len(documents) == 0:
+            documents = self.homepage.primary_documents
+        return documents
+
+    @property
+    def period_of_report(self):
+        """
+        Get the period of report for the filing
+        """
+        return self.homepage.period_of_report
 
     @property
     def attachments(self):
         # Return all the attachments on the filing
-        return self.homepage.attachments
+        sgml_filing: FilingSGML = self.sgml()
+        return sgml_filing.attachments
 
     @property
     def exhibits(self):
         # Return all the exhibits on the filing
-        return self.homepage.attachments.exhibits
+        return self.attachments.exhibits
 
+    @lru_cache(maxsize=4)
     def html(self) -> Optional[str]:
         """Returns the html contents of the primary document if it is html"""
-        if self.document and not self.document.is_binary() and not self.document.empty:
-            return str(self.document.download())
+        sgml = self.sgml()
+        html = sgml.html()
+        if html and not html.startswith("<?xml"):
+            # skip PDF (for now)
+            if html.endswith("</PDF>"):
+                return None
+            if has_html_content(html):
+                return html
+            return None
+        # If the html document is not in the SGML then we have to go to the homepage
+        html = self.homepage.primary_html_document.download()
+        if isinstance(html, bytes):
+            try:
+                return html.decode("utf-8")
+            except UnicodeDecodeError:
+                return None
+        return html
 
     @lru_cache(maxsize=4)
     def xml(self) -> Optional[str]:
         """Returns the xml contents of the primary document if it is xml"""
-        xml_document = self.homepage.primary_xml_document
-        if xml_document:
-            return str(xml_document.download())
+        sgml = self.sgml()
+        return sgml.xml()
 
     @lru_cache(maxsize=4)
     def text(self) -> str:
@@ -1276,13 +1321,18 @@ class Filing:
         html_content = self.html()
         if html_content and has_html_content(html_content):
             document = Document.parse(html_content)
-            return rich_to_text(document, 200)
+            return rich_to_text(document)
         else:
-            return self._download_filing_text()
+            text_extract_attachments = self.attachments.query("document_type == 'TEXT-EXTRACT'")
+            if len(text_extract_attachments) > 0 and text_extract_attachments.get_by_index(0) is not None:
+                text_extract_attachment = text_extract_attachments.get_by_index(0)
+                return text_extract_attachment.content
+            else:
+                return self._download_filing_text()
 
     def _download_filing_text(self):
         """
-        Download the text of the filing directly from the primary tesxt sources.
+        Download the text of the filing directly from the primary text sources.
         Either from the text url or the text extract attachment
         """
         text_extract_attachments = self.attachments.query("document_type == 'TEXT-EXTRACT'")
@@ -1293,7 +1343,6 @@ class Filing:
         else:
             return download_text_between_tags(self.text_url, "TEXT")
 
-
     def full_text_submission(self) -> str:
         """Return the complete text submission file"""
         downloaded = download_file(self.text_url, as_text=True)
@@ -1354,18 +1403,43 @@ class Filing:
         with path.open("rb") as file:
             return pickle.load(file)
 
-    @property
-    @lru_cache(maxsize=1)
+    @cached_property
     def filing_directory(self) -> FilingDirectory:
         return FilingDirectory.load(self.base_dir)
 
-    @cached_property
-    def filing_sgml(self) -> str:
-        return FilingSgml.from_filing(self)
+    @lru_cache()
+    def sgml(self) -> FilingSGML:
+        """
+        Read the filing from the local storage path if it exists
+        """
+        if is_using_local_storage():
+            local_path = local_filing_path(str(self.filing_date), self.accession_no)
+            if local_path.exists():
+                return FilingSGML.from_source(local_path)
+        return FilingSGML.from_filing(self)
 
-    @property
-    @lru_cache(maxsize=1)
+    @cached_property
+    def reports(self)  -> Optional[Reports]:
+        """
+        If the filing has report attachments then return the reports
+        """
+        filing_summary = self.sgml().filing_summary
+        if filing_summary:
+            return filing_summary.reports
+
+    @cached_property
+    def statements(self) -> Optional[Statements]:
+        """
+        Get the statements for a report
+        """
+        if self.reports:
+            return self.reports.statements
+
+    @cached_property
     def index_headers(self) -> IndexHeaders:
+        """
+        Get the index headers for the filing. This is a listing of all the files in the filing directory
+        """
         index_headers_url = f"{self.base_dir}/{self.accession_no}-index-headers.html"
         index_header_text = download_text(index_headers_url)
         return IndexHeaders.load(index_header_text)
@@ -1401,6 +1475,11 @@ class Filing:
     @property
     @lru_cache(maxsize=1)
     def header(self):
+        if is_using_local_storage():
+            _sgml = self.sgml()
+            if _sgml:
+                return _sgml.header
+
         sec_header_content = download_text_between_tags(self.text_url, "SEC-HEADER")
         try:
             return FilingHeader.parse_from_sgml_text(sec_header_content)
@@ -1408,6 +1487,8 @@ class Filing:
             # Try again with preprocessing - likely with a filing from the 1990's
             return FilingHeader.parse_from_sgml_text(sec_header_content, preprocess=True)
 
+
+
     def data_object(self):
         """ Get this filing as the data object that it might be"""
         from edgar import obj
@@ -1424,7 +1505,8 @@ class Filing:
     def open(self):
         """Open the main filing document"""
         assert self.document is not None
-        webbrowser.open(self.document.url)
+        # Use the homepage to determine the url since SGML sometimes miss the primary HTML file
+        webbrowser.open(self.homepage.primary_html_document.url)
 
     @lru_cache(maxsize=1)
     def sections(self) -> List[str]:
@@ -1448,6 +1530,10 @@ class Filing:
             return self.__get_regex_search_index().search(query)
         return self.__get_bm25_search_index().search(query)
 
+    @property
+    def filing_url(self) -> str:
+        return f"{self.base_dir}/{self.document.document}"
+
     @property
     def homepage_url(self) -> str:
         return f"{sec_edgar}/data/{self.cik}/{self.accession_no}-index.html"
@@ -1513,10 +1599,23 @@ class Filing:
             return
 
         filings = company.get_filings(accession_number=self.accession_no)
-        if filings and not filings.empty:
-            file_number = filings[0].file_number
-            return company.get_filings(file_number=file_number,
-                                       sort_by=[("filing_date", "ascending"), ("accession_number", "ascending")])
+        if not filings or filings.empty:
+            if is_using_local_storage():
+                # In this case the local storage is missing the filing so we have to download it
+                log.warning(f"Filing {self.accession_no} not found in local storage. Downloading from SEC ...")
+                from edgar.entities import download_entity_submissions_from_sec, parse_entity_submissions
+                submissions_json = download_entity_submissions_from_sec(self.cik)
+                c_from_sec = parse_entity_submissions(submissions_json)
+                filings = c_from_sec.get_filings(accession_number=self.accession_no)
+
+                if not filings or filings.empty:
+                    # Shouldn't get here
+                    return company.get_empty_filings()
+            else:
+                return company.get_empty_filings()
+        file_number = filings[0].file_number
+        return company.get_filings(file_number=file_number,
+                                   sort_by=[("filing_date", "ascending"), ("accession_number", "ascending")])
 
     def __hash__(self):
         return hash(self.accession_no)
@@ -1555,27 +1654,35 @@ class Filing:
         └──────────────────────┴──────┴────────────┴────────────────────┴─────────┘
         :return: a rich table version of this filing
         """
-        summary_table = Table(box=box.ROUNDED, show_header=False)
-        summary_table.add_column("Accession#", style="bold deep_sky_blue1", header_style="bold")
-        summary_table.add_column("Filed")
-        summary_table.add_row(self.accession_no, str(self.filing_date))
+        ticker = find_ticker(self.cik)
+        ticker = f"{ticker}" if ticker else ""
 
-        homepage_url = Text(f"\U0001F3E0 {self.homepage_url.replace('//www.', '//')}")
-        primary_doc_url = Text(f"\U0001F4C4 {self.document.url.replace('//www.', '//') if self.document else ''}")
-        submission_text_url = Text(f"\U0001F4DC {self.text_url.replace('//www.', '//')}")
-
-        links_table = Table(
-            "[b]Links[/b]: \U0001F3E0 Homepage \U0001F4C4 Primary Document \U0001F4DC Full Submission Text",
-            box=box.ROUNDED)
-        links_table.add_row(homepage_url)
-        links_table.add_row(primary_doc_url)
-        links_table.add_row(submission_text_url)
+        # The title of the panel
+        title = Text.assemble((f"Form {self.form} ", "bold"),
+                              (self.company, "bold green"),
+                              " ",
+                              (f"[{self.cik}] ", "dim"),
+                              (ticker, "bold yellow")
+                              )
+        # The subtitle of the panel
+        subtitle = Text(describe_form(self.form, False), "dim")
 
+        attachments = self.attachments
+        # The filing information table
+        filing_info_table = Table("Accession Number", "Filing Date", "Period of Report", "Documents",
+                                  header_style="dim",
+                                  box=box.SIMPLE_HEAD)
+        filing_info_table.add_row(Text(self.accession_no, "bold deep_sky_blue1"),
+                                  Text(str(self.filing_date), "bold"),
+                                  Text(self.period_of_report or "-", "bold"),
+                                  f"{len(attachments)}")
         return Panel(
-            Group(summary_table, links_table),
-            title=Text(f"{self.company} [{self.cik}] {self.form} {unicode_for_form(self.form)}", style="bold"),
-            subtitle=describe_form(self.form),
-            box=box.ROUNDED
+            Group(filing_info_table),
+            title=title,
+            subtitle=subtitle,
+            box=box.ROUNDED,
+            height=10,
+            expand=False
         )
 
     def __repr__(self):
@@ -1662,11 +1769,92 @@ page_facing_up = '\U0001F4C4'
 classical_building = '\U0001F3DB'
 
 
-def unicode_for_form(form: str):
-    if form in ['10-K', '10-Q', '10-K/A', '10-Q/A', '6-K', '6-K/A']:
-        return barchart
+def unicode_for_form(form: str) -> str:
+    """
+    Returns a meaningful Unicode symbol based on SEC form type.
+
+    Args:
+        form (str): SEC form type identifier
+
+    Returns:
+        str: Unicode symbol representing the form type
+
+    Form type categories:
+    - Periodic Reports (10-K, 10-Q): 📊 (financial statements/data)
+    - Current Reports (8-K, 6-K): ⚡ (immediate/material events)
+    - Registration & Offerings:
+        - S-1, F-1: 🎯 (initial public offerings)
+        - S-3, F-3: 🔄 (follow-on offerings)
+        - Prospectuses (424B*): 📖 (offering documents)
+    - Insider Forms (3, 4, 5): 👥 (insider activity)
+    - Beneficial Ownership:
+        - SC 13D/G: 🏰 (significant ownership stakes)
+        - 13F-HR: 📈 (institutional holdings)
+    - Investment Company:
+        - N-CSR, N-Q: 💼 (investment portfolio reports)
+        - N-PX: 🗳️ (proxy voting record)
+    - Foreign Company Forms (20-F, 40-F): 🌐 (international)
+    - Municipal Advisor Forms (MA): ⚖️ (regulation/compliance)
+    - Communications (CORRESP/UPLOAD): 💬 (dialogue with SEC)
+    - Proxy Materials (DEF 14A): 📩 (shareholder voting)
+    - Default: 📄 (generic document)
+    """
+
+    # Periodic financial reports
+    if form in ['10-K', '10-Q', '10-K/A', '10-Q/A']:
+        return '📊'  # Chart for financial statements
+
+    # Current reports (material events)
+    elif form in ['8-K', '8-K/A', '6-K', '6-K/A']:
+        return '⚡'  # Lightning bolt for immediate/current events
+
+    # Initial registration statements
+    elif form.startswith(('S-1', 'F-1')) or form in ['S-1/A', 'F-1/A']:
+        return '🎯'  # Target for initial offerings
+
+    # Shelf registration statements
+    elif form.startswith(('S-3', 'F-3')) or form in ['S-3/A', 'F-3/A']:
+        return '🔄'  # Circular arrows for repeat/follow-on offerings
+
+    # Prospectuses
+    elif form.startswith('424B'):
+        return '📖'  # Open book for offering documents
+
+    # Foreign issuer annual reports
+    elif form in ['20-F', '20-F/A', '40-F', '40-F/A']:
+        return '🌐'  # Globe for international filings
+
+    # Insider trading forms
     elif form in ['3', '4', '5', '3/A', '4/A', '5/A']:
-        return ticket
-    elif form in ['MA-I', 'MA-I/A', 'MA', 'MA/A']:
-        return classical_building
-    return page_facing_up
+        return '👥'  # People for insider/beneficial owner reports
+
+    # Significant beneficial ownership reports
+    elif form.startswith(('SC 13D', 'SC 13G')) or form in ['SC 13D/A', 'SC 13G/A']:
+        return '🏰'  # Castle for large ownership stakes
+
+    # Institutional investment holdings
+    elif form in ['13F-HR', '13F-HR/A', '13F-NT', '13F-NT/A']:
+        return '📈'  # Chart up for investment positions
+
+    # Investment company reports
+    elif form in ['N-CSR', 'N-CSR/A', 'N-Q', 'N-Q/A']:
+        return '💼'  # Briefcase for investment portfolio
+
+    # Proxy voting records
+    elif form in ['N-PX', 'N-PX/A']:
+        return '🗳️'  # Ballot box for voting records
+
+    # Municipal advisor forms
+    elif form in ['MA', 'MA/A', 'MA-I', 'MA-I/A']:
+        return '⚖️'  # Scales for regulatory/compliance
+
+    # SEC correspondence
+    elif form in ['CORRESP', 'UPLOAD']:
+        return '💬'  # Speech bubble for communications
+
+    # Proxy statements
+    elif form in ['DEF 14A', 'PRE 14A', 'DEFA14A', 'DEFC14A']:
+        return '📩'  # Envelope for shareholder communications
+
+    # Default case - generic document
+    return '📄'
