@@ -11,25 +11,28 @@ from functools import lru_cache
 from pathlib import Path
 from threading import Thread
 from typing import List, Optional, Tuple
-from typing import Union
+from typing import Union, Dict
 
 from bs4 import BeautifulSoup
-from pydantic import BaseModel, field_validator
+from pydantic import BaseModel
 from rich import box
 from rich.columns import Columns
 from rich.console import Group
 from rich.panel import Panel
 from rich.table import Table, Column
 from rich.text import Text
+import textwrap
 
 
-from edgar.richtools import repr_rich, print_xml, print_rich
-from edgar.core import sec_dot_gov, display_size, binary_extensions, text_extensions, has_html_content
+from edgar.richtools import repr_rich, print_xml, print_rich, rich_to_text
+from edgar.core import sec_dot_gov, binary_extensions, text_extensions, has_html_content
 from edgar.httprequests import get_with_retry, download_file, download_file_async
+from edgar.httpclient import async_http_client
+
 
 xbrl_document_types = ['XBRL INSTANCE DOCUMENT', 'XBRL INSTANCE FILE', 'EXTRACTED XBRL INSTANCE DOCUMENT']
 
-__all__ = ['Attachment', 'Attachments', 'FilingHomepage', 'FilerInfo', 'AttachmentServer', 'sec_document_url']
+__all__ = ['Attachment', 'Attachments', 'FilingHomepage', 'FilerInfo', 'AttachmentServer', 'sec_document_url', 'get_document_type']
 
 
 def sec_document_url(attachment_url: str) -> str:
@@ -37,6 +40,106 @@ def sec_document_url(attachment_url: str) -> str:
     attachment_url = re.sub(r"ix(\.xhtml)?\?doc=/", "", attachment_url)
     return f"{sec_dot_gov}{attachment_url}"
 
+def sequence_sort_key(x):
+    seq = x.sequence_number
+    if seq.strip() == '':  # Handle empty or whitespace-only strings
+        return (float('inf'), '')  # Sort to end using infinity
+    try:
+        return (0, float(seq))  # Convert to number for numeric sorting
+    except ValueError:
+        return (1, seq)  #
+
+
+# Mapping of SEC filing file types to Unicode symbols
+FILE_TYPE_SYMBOLS: Dict[str, str] = {
+    # Main SEC filing documents
+    "10-K": "📄",     # Document emoji for main filing
+    "EX-21.1": "📎",  # Paperclip for exhibits
+    "EX-23.1": "📎",
+    "EX-31.1": "📎",
+    "EX-31.2": "📎",
+    "EX-32.1": "📎",
+    "EX-97.1": "📎",
+
+    # XBRL-related documents
+    "EX-101.SCH": "🔰",  # Clipboard for schema
+    "EX-101.CAL": "📊",  # Chart for calculations
+    "EX-101.DEF": "📚",  # Books for definitions
+    "EX-101.LAB": "📎",  # Paperclip for labels (changed from label)
+    "EX-101.PRE": "📈",  # Graph for presentation
+
+    # Common file types
+    "XML": "🔷",      # Document for XML files
+    "HTML": "🌍",     # Page for HTML files
+    "GRAPHIC": "🎨",  # Camera for images
+    "EXCEL": "📊",    # Chart for Excel
+    "JSON": "📝",     # Note for JSON
+    "ZIP": "📦",      # Package for ZIP
+    "CSS": "📃",      # Page for CSS
+    "JS": "📄",       # Document for JavaScript
+    ".css": "📃",     # Page for CSS extension
+    ".js": "📄",      # Document for JS extension
+    "PDF": "📕",      # Book for PDF
+    ".pdf": "📕",     # Book for PDF extension
+    "INFORMATION TABLE": "📊"  # Chart for tables
+}
+
+
+def get_extension(filename: str) -> str:
+    """Extract the file extension including the dot."""
+    if '.' in filename:
+        return filename[filename.rindex('.'):]
+    return ''
+
+def get_document_type(filename: str, declared_document_type:str) -> str:
+    """
+    Sometimes the SEC gets the document type wrong. This function uses the extension to determine the document type
+    """
+    if declared_document_type.upper() in ["XML", "HTML", "PDF", "HTM",  "JS", "CSS", "ZIP", "XLS", "XSLX", "JSON"]:
+        extension = get_extension(filename)
+        document_type = extension[1:].upper()
+        if document_type in ["HTM", "HTML"]:
+            return "HTML"
+        return document_type
+    return declared_document_type
+
+def get_file_icon(file_type: str, sequence: str = None, filename: str = None) -> str:
+    """
+    Get the Unicode symbol for a given file type and sequence number.
+
+    Args:
+        file_type: The type of the file from SEC filing
+        sequence: The sequence number of the file in the filing
+        filename: The name of the file to extract the extension
+
+    Returns:
+        Unicode symbol corresponding to the file type.
+        If sequence is 1, returns "📜" (scroll) to indicate main filing document.
+        Returns "📄" (document) as default if type not found.
+    """
+    icon = None
+    if sequence == "1":
+        icon = "📜"  # Scroll emoji for main document
+
+    # Check if it's an XBRL exhibit (EX-101.*)
+    elif file_type.startswith("EX-101."):
+        icon = FILE_TYPE_SYMBOLS.get(file_type, "📄")
+
+    # Check if it's a regular exhibit (starts with EX-)
+    elif file_type.startswith("EX-"):
+        icon = "📋"  # Clipboard + writing hand for exhibits
+
+    # Check for file extension first if filename is provided
+    elif filename:
+        ext = get_extension(filename)
+        if ext in FILE_TYPE_SYMBOLS:
+            icon = FILE_TYPE_SYMBOLS[ext]
+
+    if not icon:
+        icon =FILE_TYPE_SYMBOLS.get(file_type, "📄")
+    icon = f"{icon} " if len(icon) == 1 else icon # Add spaces around the icon for padding
+    return icon
+
 
 class FilerInfo(BaseModel):
     company_name: str
@@ -54,17 +157,39 @@ class FilerInfo(BaseModel):
         return repr_rich(self.__rich__())
 
 
-class Attachment(BaseModel):
+class Attachment:
     """
     A class to represent an attachment in an SEC filing
     """
-    sequence_number: str
-    description: str
-    document: str
-    ixbrl: bool
-    path: str
-    document_type: str
-    size: Optional[int]
+
+    def __init__(self,
+                 sequence_number: str,
+                 description: str,
+                 document: str,
+                 ixbrl: bool,
+                 path: str,
+                 document_type: str,
+                 size: Optional[int],
+                 sgml_document: Optional['SGMLDocument'] = None,
+                 purpose: Optional[str] = None,
+                 filing_sgml: Optional['FilingSGML'] = None):
+        self.sequence_number = sequence_number
+        self.description = description
+        self.document = document
+        self.ixbrl = ixbrl
+        self.path = path
+        self.document_type = document_type
+        self.size = size
+        self.sgml_document:Optional['SGMLDocument'] = sgml_document
+        self.sgml = filing_sgml
+        self.purpose = purpose
+
+    @property
+    def content(self):
+        if self.sgml_document:
+            return self.sgml_document.content
+        else:
+            return download_file(self.url)
 
     @property
     def url(self):
@@ -75,7 +200,7 @@ class Attachment(BaseModel):
         """The actual extension of the filing document
          Usually one of .xml or .html or .pdf or .txt or .paper
          """
-        return os.path.splitext(self.path)[1]
+        return os.path.splitext(self.document)[1]
 
     @property
     def display_extension(self) -> str:
@@ -84,7 +209,6 @@ class Attachment(BaseModel):
         """
         return os.path.splitext(self.document)[1]
 
-    @field_validator('sequence_number')
     def validate_sequence_number(cls, v):
         if not v.isdigit() and v != '':
             raise ValueError('sequence_number must be digits or an empty string')
@@ -116,9 +240,8 @@ class Attachment(BaseModel):
             If the path is a directory, the file is saved with its original name in that directory.
             If the path is a file, the file is saved with the given path name.
             """
-        downloaded = download_file(self.url, as_text=self.is_text())
         if path is None:
-            return downloaded
+            return self.content
 
         # Ensure path is a Path object
         path = Path(path)
@@ -130,32 +253,66 @@ class Attachment(BaseModel):
             file_path = path
 
         # Save the file
-        if isinstance(downloaded, bytes):
-            file_path.write_bytes(downloaded)
+        if isinstance(self.content, bytes):
+            file_path.write_bytes(self.content)
         else:
-            file_path.write_text(downloaded)
+            file_path.write_text(self.content)
 
         return str(file_path)
 
     def view(self):
+        # Check if this is a report
+        if self.is_report() and self.sgml:
+            report = self.sgml.filing_summary.reports.get_by_filename(self.document)
+            if report:
+                report.view()
+        else:
+            if self.is_text():
+                content = self.content
+                if self.is_html() or has_html_content(content):
+                    from edgar import Document
+                    document = Document.parse(content)
+                    print_rich(document)
+                elif self.is_xml():
+                    print_xml(content)
+                else:
+                    print(content)
+            else:
+                print(self)
+
+    def is_report(self):
+        return re.match(r"R\d+\.htm", self.document)
+
+    def text(self):
+        # Check if this is a report
+        if self.is_report() and self.sgml:
+            report = self.sgml.filing_summary.reports.get_by_filename(self.document)
+            if report:
+                return report.text()
+
         if self.is_text():
-            content = self.download()
+            content = self.content
             if self.is_html() or has_html_content(content):
                 from edgar import Document
                 document = Document.parse(content)
-                print_rich(document)
-            elif self.is_xml():
-                print_xml(content)
+                return rich_to_text(document)
             else:
-                print(content)
-        else:
-            print(self)
+                return content
+        return None
 
     def __rich__(self):
-        table = Table("Document", "Description", "Type", "Size", box=box.ROUNDED)
-        table.add_row(self.document, self.description, self.document_type,
-                      display_size(self.size))
-        return table
+        icon = get_file_icon(self.document_type, self.sequence_number, self.document)
+        text = Text.assemble( (f"{self.sequence_number:<3} ", "dim italic"),
+                             " ",
+                             (self.document, "bold"),
+                             " ", (self.purpose or self.description, "grey54"),
+                             " ",
+                             (icon, ""),
+                              " ",
+                              (self.document_type,
+                               "bold deep_sky_blue1" if self.sequence_number == "1" else "")
+                             )
+        return Panel(text, box=box.ROUNDED, width=200, expand=False)
 
     def __repr__(self):
         return repr_rich(self.__rich__())
@@ -172,15 +329,22 @@ class Attachments:
     def __init__(self,
                  document_files: List[Attachment],
                  data_files: Optional[List[Attachment]],
-                 primary_documents: List[Attachment]):
+                 primary_documents: List[Attachment],
+                 sgml:Optional['FilingSGML'] = None):
         self.documents = document_files
         self.data_files = data_files
         self._attachments = document_files + (data_files or [])
         self.primary_documents = primary_documents
+        self.sgml = sgml
+        self.n = 0
+
 
     def __getitem__(self, item: Union[int, str]):
-        if isinstance(item, int):
-            return self._attachments[item]
+        """
+        Get the attachment by sequence number as set in the SEC filing SGML file
+        """
+        if isinstance(item, int) or item.isdigit():
+            return self.get_by_sequence(item)
         elif isinstance(item, str):
             for doc in self._attachments:
                 if doc.document == item:
@@ -188,19 +352,46 @@ class Attachments:
         raise KeyError(f"Document not found: {item}")
 
     def get_by_sequence(self, sequence: Union[str, int]):
+        """
+        Get the attachment by sequence number starting at 1
+        The sequence number is the exact sequence number in the filing
+        """
         for doc in self._attachments:
             if doc.sequence_number == str(sequence):
                 return doc
         raise KeyError(f"Document not found: {sequence}")
 
+    def get_by_index(self, index: int):
+        """
+        Get the attachment by index starting at 1
+        """
+        return self._attachments[index]
+
+
+    def get_report(self, filename:str) -> 'Report':
+        """
+        Get a report by filename
+        """
+        if self.sgml:
+            reports = self.sgml.filing_summary.reports
+            if reports:
+                return reports.get_by_filename(filename)
+
+
     @property
     def primary_html_document(self) -> Optional[Attachment]:
         """Get the primary xml document on the filing"""
         for doc in self.primary_documents:
             if doc.display_extension == ".html" or doc.display_extension == '.htm':
                 return doc
-        # Shouldn't get here but just open the first document
-        return self.primary_documents[0]
+        """
+        Most filings have html primary documents. Some don't. 
+        E.g. Form's 3,4,5 do when loaded directly from edgar but not when loaded from local files
+        However, there are unusual filings with endings like ".fil" that require a return. So return the first one
+        """
+        if len(self.primary_documents) > 0:
+            return self.primary_documents[0]
+
 
     @property
     def primary_xml_document(self) -> Optional[Attachment]:
@@ -225,7 +416,10 @@ class Attachments:
         primary_documents = [self.primary_html_document]
         exhibits_documents = self.query("re.match('EX-', document_type)", False).documents
         return Attachments(
-            document_files=primary_documents + exhibits_documents, data_files=[], primary_documents=primary_documents)
+            document_files=primary_documents + exhibits_documents,
+            data_files=[],
+            primary_documents=primary_documents,
+            sgml=self.sgml)
 
     @property
     def graphics(self):
@@ -265,13 +459,16 @@ class Attachments:
             new_data_files = []
 
         return Attachments(document_files=new_documents, data_files=new_data_files,
-                           primary_documents=self.primary_documents)
+                           primary_documents=self.primary_documents, sgml=self.sgml)
 
     @staticmethod
     async def _download_all_attachments(attachments: List[Attachment]):
         import asyncio
-        return await asyncio.gather(
-            *[download_file_async(attachment.url, as_text=attachment.is_text()) for attachment in attachments])
+
+        async with async_http_client() as client:
+            return await asyncio.gather(
+                *[download_file_async(client, attachment.url, as_text=attachment.is_text()) for attachment in attachments])
+
 
     def download(self, path: Union[str, Path], archive: bool = False):
         """
@@ -282,6 +479,9 @@ class Attachments:
         path: str or Path - The path to save the attachments
         archive: bool (default False) - If True, save the attachments in a zip file
         """
+        if self.sgml:
+            self.sgml.download(path, archive)
+            return
 
         import asyncio
         loop = asyncio.get_event_loop()
@@ -291,25 +491,27 @@ class Attachments:
         path = Path(path)
 
         # If the path is a directory, save the files in that directory
-        if path.is_dir():
-            for attachment, downloaded in zip(self._attachments, downloaded_files):
-                file_path = path / attachment.document
-                if isinstance(downloaded, bytes):
-                    file_path.write_bytes(downloaded)
-                else:
-                    file_path.write_text(downloaded, encoding='utf-8')
-
-        # If the path is an archive file, save the files in that file
-        else:
-            if archive:
+        if archive:
+            if path.is_dir():
+                raise ValueError("Path must be a zip file name to create zipfile")
+            else:
                 with zipfile.ZipFile(path, 'w') as zipf:
                     for attachment, downloaded in zip(self._attachments, downloaded_files):
                         if isinstance(downloaded, bytes):
                             zipf.writestr(attachment.document, downloaded)
                         else:
                             zipf.writestr(attachment.document, downloaded.encode('utf-8'))
+        else:
+            if path.is_dir():
+                for attachment, downloaded in zip(self._attachments, downloaded_files):
+                    file_path = path / attachment.document
+                    if isinstance(downloaded, bytes):
+                        file_path.write_bytes(downloaded)
+                    else:
+                        file_path.write_text(downloaded, encoding='utf-8')
             else:
-                raise ValueError("Path must be a directory or an archive file")
+                raise ValueError("Path must be a directory")
+
 
     def serve(self, port: int = 8000) -> Tuple[Thread, socketserver.TCPServer, str]:
         """
@@ -378,36 +580,31 @@ class Attachments:
     def __rich__(self):
 
         # Document files
-        document_table = Table('Seq', Column('Document'), 'Description', 'Type', 'Size',
-                               title='Documents',
+        document_table = Table(Column('Seq', header_style="dim"),
+                                        Column('Document', header_style="dim"),
+                                        Column('Description', header_style="dim", min_width=60),
+                                        Column('Type', header_style="dim", min_width=16),
+                               title='Attachments',
                                row_styles=["", "bold"],
-                               box=box.SIMPLE)
-        for index, _attachment in enumerate(self.documents):
-            document_table.add_row(str(_attachment.sequence_number),
-                                   _attachment.document,
-                                   _attachment.description,
-                                   _attachment.document_type,
-                                   display_size(_attachment.size))
-        document_panel = Panel(document_table, box=box.ROUNDED)
+                               box=box.ROUNDED)
+        all_attachments = sorted(self.documents + (self.data_files or []), key=sequence_sort_key)
 
-        renderables = [document_panel]
 
-        # Data files
-        if self.data_files:
-            data_table = Table('Seq', Column('Document'), 'Description', 'Type', 'Size',
-                               title='Data Files',
-                               row_styles=["", "bold"],
-                               box=box.SIMPLE)
-            for index, _attachment in enumerate(self.data_files):
-                data_table.add_row(str(_attachment.sequence_number),
-                                   _attachment.document,
-                                   _attachment.description,
-                                   _attachment.document_type,
-                                   display_size(_attachment.size))
-            data_panel = Panel(data_table, box=box.ROUNDED)
-            renderables.append(data_panel)
 
-        return Group(*renderables)
+        for attachment in all_attachments:
+            # Get the file icon for each attachment
+            icon = get_file_icon(file_type=attachment.document_type,
+                                 sequence= attachment.sequence_number,
+                                 filename=attachment.document)
+            sequence_number = f"{attachment.sequence_number}" if attachment.sequence_number == "1" else attachment.sequence_number
+            description = "\n".join(textwrap.wrap(attachment.purpose or attachment.description, 100))
+            document_table.add_row(Text(sequence_number, style="bold deep_sky_blue1") if attachment.sequence_number == "1" else sequence_number,
+                                   Text(attachment.document, style="bold deep_sky_blue1") if attachment.sequence_number == "1" else attachment.document,
+                                   Text(description, style="bold deep_sky_blue1") if attachment.sequence_number == "1" else description,
+                                   Text.assemble((icon, ""), " ", (attachment.document_type, "bold deep_sky_blue1" if attachment.sequence_number == "1" else "")),)
+
+
+        return document_table
 
     def __repr__(self):
         return repr_rich(self.__rich__())
@@ -456,6 +653,9 @@ class Attachments:
                 )
                 # Add the attachment to the list
                 attachments.append(attachment)
+
+                # Set the SGML on the attachment
+                attachment.sgml = attachment.sgml
                 # If this is the first document, set it as the primary document
                 if documents:
                     if min_seq is None:
@@ -554,7 +754,7 @@ class FilingHomepage:
 
     @property
     def primary_html_document(self) -> Optional[Attachment]:
-        """Get the primary xml document on the filing"""
+        """Get the primary html document on the filing"""
         return self.attachments.primary_html_document
 
     @property
@@ -627,16 +827,30 @@ class FilingHomepage:
 
         return filer_infos
 
+    @property
+    def period_of_report(self)-> Optional[str]:
+        "Get the period of report"
+        _,_, period = self.get_filing_dates()
+        return period
+
     @lru_cache(maxsize=None)
-    def get_filing_dates(self)-> Optional[Tuple[str,str]]:
-        # Find the div with the formGrouping class
-        form_grouping = self._soup.find("div", class_="formGrouping")
-        if form_grouping is None:
+    def get_filing_dates(self)-> Optional[Tuple[str,str, Optional[str]]]:
+        # Find the form grouping divs
+        grouping_divs = self._soup.find_all("div", class_="formGrouping")
+        if len(grouping_divs) == 0:
             return None
-        info_divs = form_grouping.find_all("div", class_="info")
+        date_grouping_div = grouping_divs[0]
+        info_divs = date_grouping_div.find_all("div", class_="info")
         filing_date = info_divs[0].text.strip()
         accepted_date = info_divs[1].text.strip()
-        return filing_date, accepted_date
+
+        if len(grouping_divs) > 1:
+            period_grouping_div = grouping_divs[1]
+            first_info_div = period_grouping_div.find("div", class_="info")
+            if first_info_div:
+                period = first_info_div.text.strip()
+                return filing_date, accepted_date, period
+        return filing_date, accepted_date, None
 
     @classmethod
     def load(cls, url: str):
