@@ -9,7 +9,6 @@ from rich.padding import Padding
 from rich.panel import Panel
 from rich.table import Table
 from rich.tree import Tree
-from rich.layout import Layout
 
 from edgar._filings import Attachments, Attachment
 from edgar._markdown import MarkdownContent
@@ -18,13 +17,14 @@ from edgar.files.html import Document
 from edgar.files.html_documents import HtmlDocument
 from edgar.files.htmltools import ChunkedDocument, chunks2df, detect_decimal_items, adjust_for_empty_items
 from edgar.financials import Financials
-from edgar.richtools import repr_rich
-from edgar.richtools import rich_to_text
+from edgar.richtools import repr_rich, rich_to_text
 
 __all__ = [
     'TenK',
     'TenQ',
     'TwentyF',
+    'CurrentReport',
+    'SixK',
     'EightK',
     'PressRelease',
     'PressReleases',
@@ -491,7 +491,7 @@ class TwentyF(CompanyReport):
         return f"""TwentyF('{self.company}')"""
 
 
-class EightK():
+class CurrentReport():
     structure = ItemOnlyFilingStructure({
         "ITEM 1.01": {
             "Title": "Entry into a Material Definitive Agreement",
@@ -574,7 +574,7 @@ class EightK():
     })
 
     def __init__(self, filing):
-        assert filing.form in ['8-K', '8-K/A'], f"This form should be an 8-K but was {filing.form}"
+        assert filing.form in ['8-K', '8-K/A', '6-K', '6-K/A'], f"This form should be an 8-K but was {filing.form}"
         self._filing = filing
 
     @property
@@ -658,7 +658,7 @@ class EightK():
         # from the sgml content
         if exhibit.empty:
             # Download the SGML document
-            sgml_document = self._filing.filing_sgml.get_by_sequence(exhibit.sequence_number)
+            sgml_document = self._filing.sgml().get_document_by_sequence(exhibit.sequence_number)
             if sgml_document:
                 exhibit_content = sgml_document.text()
                 return exhibit_content
@@ -666,7 +666,7 @@ class EightK():
             html_content = exhibit.download()
             if html_content:
                 document = Document.parse(html_content)
-                return rich_to_text(document, 200)
+                return repr_rich(document, width=200, force_terminal=False)
 
     def _content_renderables(self):
         """Get the content of the exhibits as renderables"""
@@ -682,7 +682,7 @@ class EightK():
                 renderables.append(Panel(exhibit_content,
                                          title=title,
                                          subtitle=Text(exhibit.description, style="gray54"),
-                                         box=box.ROUNDED))
+                                         box=box.SIMPLE))
         return Group(*renderables)
 
     def text(self):
@@ -716,7 +716,8 @@ class EightK():
 
         return Panel(
             Group(*renderables),
-            title=panel_title
+            title=panel_title,
+            box=box.SIMPLE
         )
 
     def __str__(self):
@@ -725,6 +726,9 @@ class EightK():
     def __repr__(self):
         return repr_rich(self.__rich__())
 
+# Aliases fpr the current report
+EightK = CurrentReport
+SixK = CurrentReport
 
 class PressReleases:
     """
@@ -738,7 +742,7 @@ class PressReleases:
         return len(self.attachments)
 
     def __getitem__(self, item):
-        attachment = self.attachments[item]
+        attachment = self.attachments.get_by_index(item)
         if attachment:
             return PressRelease(attachment)
 
