@@ -11,10 +11,11 @@ from _thread import interrupt_main
 from dataclasses import dataclass
 from decimal import Decimal
 from functools import lru_cache
-from pathlib import Path
-from typing import Union, Optional, Tuple, List
 from functools import wraps
-
+from typing import Union, Optional, Tuple, List
+from pathlib import Path
+from datetime import date
+import pytz
 import httpx
 import humanize
 import pandas as pd
@@ -24,15 +25,7 @@ from pandas.tseries.offsets import BDay
 from rich.logging import RichHandler
 from rich.prompt import Prompt
 
-# Rich logging
-logging.basicConfig(
-    level="INFO",
-    format="%(message)s",
-    datefmt="[%X]",
-    handlers=[RichHandler(rich_tracebacks=True)]
-)
-
-log = logging.getLogger("rich")
+log = logging.getLogger(__name__)
 
 # Pandas version
 pandas_version = tuple(map(int, pd.__version__.split('.')))
@@ -40,9 +33,6 @@ pandas_version = tuple(map(int, pd.__version__.split('.')))
 # sys version
 python_version = tuple(map(int, sys.version.split()[0].split('.')))
 
-# Turn down 3rd party logging
-logging.getLogger("httpx").setLevel(logging.WARNING)
-
 __all__ = [
     'log',
     'Result',
@@ -58,7 +48,6 @@ __all__ = [
     'IntString',
     'DataPager',
     'yes_no',
-    'http_client',
     'sec_dot_gov',
     'display_size',
     'reverse_name',
@@ -68,12 +57,14 @@ __all__ = [
     'pandas_version',
     'python_version',
     'set_identity',
+    'strtobool',
     'listify',
     'decode_content',
     'filter_by_date',
     'filter_by_form',
     'filter_by_cik',
     'filter_by_ticker',
+    'filter_by_exchange',
     'filter_by_accession_number',
     'split_camel_case',
     'cache_except_none',
@@ -81,22 +72,32 @@ __all__ = [
     'binary_extensions',
     'ask_for_identity',
     'is_start_of_quarter',
-    'use_local_storage',
-    'is_using_local_storage',
     'run_async_or_sync',
-    'has_html_content',
-    'download_edgar_data',
     'get_edgar_data_directory',
+    'has_html_content',
     'default_page_size',
+    'parse_acceptance_datetime',
     'InvalidDateException',
+    'PagingState',
+    'Years',
+    'Quarters',
+    'YearAndQuarter',
+    'YearAndQuarters',
+    'quarters_in_year',
 ]
 
 IntString = Union[str, int]
+quarters_in_year: List[int] = list(range(1, 5))
+
+YearAndQuarter = Tuple[int, int]
+YearAndQuarters = List[YearAndQuarter]
+Years = Union[int, List[int], range]
+Quarters = Union[int, List[int], range]
 
 # Date patterns
 YYYY_MM_DD = "\\d{4}-\\d{2}-\\d{2}"
 DATE_PATTERN = re.compile(YYYY_MM_DD)
-DATE_RANGE_PATTERN = re.compile(f"({YYYY_MM_DD})?:?(({YYYY_MM_DD})?)?")
+DATE_RANGE_PATTERN = re.compile(f"^({YYYY_MM_DD}(:({YYYY_MM_DD})?)?|:({YYYY_MM_DD}))$")
 
 default_http_timeout: int = 12
 default_page_size = 50
@@ -106,6 +107,25 @@ default_retries = 3
 limits = httpx.Limits(max_connections=default_max_connections)
 
 
+def strtobool (val:str):
+    """Convert a string representation of truth to true (1) or false (0).
+
+    True values are case insensitive 'y', 'yes', 't', 'true', 'on', and '1'.
+    false values are case insensitive 'n', 'no', 'f', 'false', 'off', and '0'.
+    Raises ValueError if 'val' is anything else.
+    """
+    if not val:
+        return False
+    val = val.lower()
+    if val in ('y', 'yes', 't', 'true', 'on', '1'):
+        return 1
+    elif val in ('n', 'no', 'f', 'false', 'off', '0'):
+        return 0
+    else:
+        return 0
+        #raise ValueError("invalid truth value %r" % (val,))
+
+
 @dataclass
 class EdgarSettings:
     http_timeout: int
@@ -173,6 +193,9 @@ def set_identity(user_identity: str):
     os.environ[edgar_identity] = user_identity
     log.info(f"Identity of the Edgar REST client set to [{user_identity}]")
 
+    from edgar.httpclient import close_clients
+    close_clients() # close any httpx clients, to reset the identity. 
+
 
 identity_prompt = """
 [bold turquoise4]Identify your client to SEC Edgar[/bold turquoise4]
@@ -238,45 +261,10 @@ def get_edgar_data_directory() -> Path:
     """Get the edgar data directory"""
     default_local_data_dir = Path(os.path.join(os.path.expanduser("~"), ".edgar"))
     edgar_data_dir = Path(os.getenv('EDGAR_LOCAL_DATA_DIR', default_local_data_dir))
-    if not edgar_data_dir.exists():
-        os.makedirs(edgar_data_dir)
+    os.makedirs(edgar_data_dir, exist_ok=True)
     return edgar_data_dir
 
 
-def use_local_storage(use_local: bool = True):
-    """
-    Will use local data if set to True
-    """
-    os.environ['EDGAR_USE_LOCAL_DATA'] = "1" if use_local else "0"
-
-
-def is_using_local_storage() -> bool:
-    """
-    Returns True if using local storage
-    """
-    return os.getenv('EDGAR_USE_LOCAL_DATA', "0") == "1"
-
-
-def download_edgar_data(submissions: bool = True,
-                        facts: bool = True,
-                        reference: bool = True):
-    """
-    Download Edgar data to the local storage directory
-    :param submissions: Download submissions
-    :param facts: Download facts
-    :param reference: Download reference data
-    """
-    if submissions:
-        from edgar.entities import download_submissions
-        download_submissions()
-    if facts:
-        from edgar.entities import download_facts
-        download_facts()
-    if reference:
-        from edgar.reference import download_reference_data
-        download_reference_data()
-
-
 class InvalidDateException(Exception):
 
     def __init__(self, message: str):
@@ -289,38 +277,119 @@ class TooManyRequestsException(Exception):
         super().__init__(message)
 
 
-def extract_dates(date: str) -> Tuple[Optional[str], Optional[str], bool]:
+def extract_dates(date_str: str) -> Tuple[Optional[datetime.datetime], Optional[datetime.datetime], bool]:
     """
     Split a date or a date range into start_date and end_date
-        split_date("2022-03-04")
-          2022-03-04, None, False
-       split_date("2022-03-04:2022-04-05")
-        2022-03-04, 2022-04-05, True
-       split_date("2022-03-04:")
-        2022-03-04, None, True
-       split_date(":2022-03-04")
-        None, 2022-03-04, True
-    :param date: The date to split
-    :return:
+    Examples:
+        extract_dates("2022-03-04") -> 2022-03-04, None, False
+        extract_dates("2022-03-04:2022-04-05") -> 2022-03-04, 2022-04-05, True
+        extract_dates("2022-03-04:") -> 2022-03-04, <current_date>, True
+        extract_dates(":2022-03-04") -> 1994-07-01, 2022-03-04, True
+
+    Args:
+        date_str: Date string in YYYY-MM-DD format, optionally with a range separator ':'
+
+    Returns:
+        Tuple of (start_date, end_date, is_range) where dates are datetime objects
+        and is_range indicates if this was a date range query
+
+    Raises:
+        InvalidDateException: If the date string cannot be parsed
     """
-    match = re.match(DATE_RANGE_PATTERN, date)
-    if match:
-        start_date, _, end_date = match.groups()
-        try:
-            start_date_tm = datetime.datetime.strptime(start_date, "%Y-%m-%d") if start_date else None
-            end_date_tm = datetime.datetime.strptime(end_date, "%Y-%m-%d") if end_date else None
-            if start_date_tm or end_date_tm:
-                return start_date_tm, end_date_tm, ":" in date
-        except ValueError:
-            log.error(f"The date {date} cannot be extracted using date pattern YYYY-MM-DD")
-    raise InvalidDateException(f"""
-    Cannot extract a date or date range from string {date}
-    Provide either 
-        1. A date in the format "YYYY-MM-DD" e.g. "2022-10-27"
-        2. A date range in the format "YYYY-MM-DD:YYYY-MM-DD" e.g. "2022-10-01:2022-10-27"
-        3. A partial date range "YYYY-MM-DD:" to specify dates after the value e.g.  "2022-10-01:"
-        4. A partial date range ":YYYY-MM-DD" to specify dates before the value  e.g. ":2022-10-27"
-    """)
+    if not date_str:
+        raise InvalidDateException("Empty date string provided")
+
+    try:
+        # Split on colon, handling the single date case
+        has_colon = ':' in date_str
+        parts = date_str.split(':') if has_colon else [date_str]
+
+        # Handle invalid formats
+        if len(parts) != (2 if has_colon else 1):
+            raise InvalidDateException("Invalid date range format")
+
+        # Parse start date
+        if not has_colon or parts[0]:
+            start_date = datetime.datetime.strptime(parts[0], "%Y-%m-%d")
+        else:
+            start_date = datetime.datetime.strptime('1994-07-01', '%Y-%m-%d')
+
+        # Parse end date
+        if has_colon and parts[1]:
+            end_date = datetime.datetime.strptime(parts[1], "%Y-%m-%d")
+        elif has_colon:
+            end_date = datetime.datetime.now().replace(hour=0, minute=0, second=0, microsecond=0)
+        else:
+            end_date = None
+
+        # Validate date order if both dates are present
+        if has_colon and end_date and start_date > end_date:
+            raise InvalidDateException(
+                f"Invalid date range: start date ({start_date.date()}) "
+                f"cannot be after end date ({end_date.date()})"
+            )
+
+        return start_date, end_date, has_colon
+
+    except ValueError as e:
+        raise InvalidDateException(f"""
+        Cannot extract a date or date range from string {date_str}
+        Provide either 
+            1. A date in the format "YYYY-MM-DD" e.g. "2022-10-27"
+            2. A date range in the format "YYYY-MM-DD:YYYY-MM-DD" e.g. "2022-10-01:2022-10-27"
+            3. A partial date range "YYYY-MM-DD:" to specify dates after the value e.g.  "2022-10-01:"
+            4. A partial date range ":YYYY-MM-DD" to specify dates before the value  e.g. ":2022-10-27"
+        """) from e
+
+
+def filing_date_to_year_quarters(filing_date: str) -> List[Tuple[int, int]]:
+    if ":" in filing_date:
+        start_date, end_date = filing_date.split(":")
+
+        if not start_date:
+            start_date = "1994-06-01"
+
+        if not end_date:
+            end_date = date.today().strftime("%Y-%m-%d")
+
+        start_year, start_month, _ = map(int, start_date.split("-"))
+        end_year, end_month, _ = map(int, end_date.split("-"))
+
+        start_quarter = (start_month - 1) // 3 + 1
+        end_quarter = (end_month - 1) // 3 + 1
+
+        result = []
+        for year in range(start_year, end_year + 1):
+            if year == start_year and year == end_year:
+                quarters = range(start_quarter, end_quarter + 1)
+            elif year == start_year:
+                quarters = range(start_quarter, 5)
+            elif year == end_year:
+                quarters = range(1, end_quarter + 1)
+            else:
+                quarters = range(1, 5)
+
+            for quarter in quarters:
+                result.append((year, quarter))
+
+        return result
+    else:
+        year, month, _ = map(int, filing_date.split("-"))
+        quarter = (month - 1) // 3 + 1
+        return [(year, quarter)]
+
+
+def current_year_and_quarter() -> Tuple[int, int]:
+    # Define the Eastern timezone
+    eastern = pytz.timezone('America/New_York')
+
+    # Get the current time in Eastern timezone
+    now_eastern = datetime.datetime.now(eastern)
+
+    # Calculate the current year and quarter
+    current_year, current_quarter = now_eastern.year, (now_eastern.month - 1) // 3 + 1
+
+    return current_year, current_quarter
 
 
 def filter_by_date(data: pa.Table,
@@ -376,6 +445,16 @@ def filter_by_cik(data: pa.Table,
     data = data.filter(pc.is_in(data['cik'], pa.array(ciks)))
     return data
 
+def filter_by_exchange(data: pa.Table, exchange: Union[str, List[str]]) -> pa.Table:
+    """Return the data filtered by exchange"""
+    # Ensure that forms is a list of strings ... it can accept int like form 3, 4, 5
+    from edgar.reference.tickers import get_company_ticker_name_exchange
+    exchanges = [str(el).upper() for el in listify(exchange)]
+    exchange_df = get_company_ticker_name_exchange()
+    exchange_df = exchange_df[exchange_df.exchange.str.upper().isin(exchanges)]
+    return filter_by_cik(data, exchange_df.cik.tolist())
+
+
 
 def filter_by_ticker(data: pa.Table,
                      ticker: Union[str, List[str]]) -> pa.Table:
@@ -395,29 +474,6 @@ def filter_by_ticker(data: pa.Table,
 def client_headers():
     return {'User-Agent': get_identity()}
 
-
-def http_client():
-    return httpx.Client(headers=client_headers(),
-                        timeout=edgar_mode.http_timeout,
-                        limits=edgar_mode.limits,
-                        default_encoding="utf-8")
-
-
-def async_http_client():
-    return httpx.AsyncClient(headers=client_headers(),
-                             timeout=edgar_mode.http_timeout,
-                             limits=edgar_mode.limits,
-                             default_encoding='utf-8')
-
-
-def get_json(data_url: str):
-    with http_client() as client:
-        r = client.get(data_url)
-        if r.status_code == 200:
-            return r.json()
-        r.raise_for_status()
-
-
 def decode_content(content: bytes):
     try:
         return content.decode('utf-8')
@@ -430,29 +486,6 @@ binary_extensions = (".pdf", ".jpg", ".jpeg", "png", ".gif", ".tif", ".tiff", ".
                      ".apng")
 
 
-def extract_text_between_tags(content: str, tag: str) -> str:
-    """
-    Extracts text from provided content between the specified HTML/XML tags.
-
-    :param content: The text content to search through
-    :param tag: The tag to extract the content from
-    :return: The extracted text between the tags
-    """
-    tag_start = f'<{tag}>'
-    tag_end = f'</{tag}>'
-    is_tag = False
-    extracted_content = ""
-
-    for line in content.splitlines():
-        if line.startswith(tag_start):
-            is_tag = True
-            continue  # Skip the start tag line
-        elif line.startswith(tag_end):
-            break  # Stop reading if end tag is found
-        elif is_tag:
-            extracted_content += line + '\n'  # Add line to result
-
-    return extracted_content.strip()
 
 
 def repr_df(df, hide_index: bool = True):
@@ -592,6 +625,13 @@ class DataPager:
         return self.start_index + self.page_size
 
 
+@dataclass
+class PagingState:
+    page_start: int
+    num_records: int
+
+
+
 def moneyfmt(value, places=0, curr='$', sep=',', dp='.',
              pos='', neg='-', trailneg=''):
     """Convert Decimal to a money formatted string.
@@ -658,6 +698,9 @@ def datefmt(value: Union[datetime.datetime, str], fmt: str = "%Y-%m-%d") -> str:
     else:
         return value.strftime(fmt)
 
+def parse_acceptance_datetime(acceptance_datetime: str) -> datetime.datetime:
+    return datetime.datetime.fromisoformat(acceptance_datetime.replace('Z', '+00:00'))
+
 
 def sample_table(table, n=None, frac=None, replace=False, random_state=None):
     """Take a sample from a pyarrow Table"""
@@ -798,6 +841,7 @@ def is_start_of_quarter():
 
     return False
 
+
 def format_date(date: Union[str, datetime.datetime], fmt: str = "%Y-%m-%d") -> str:
     """
     Format a date as a string
@@ -870,4 +914,25 @@ def has_html_content(content: str) -> bool:
                 'xmlns:html' in first_1000):
             return True
 
-    return False
\ No newline at end of file
+    # Just check for straightforward HTML
+    if first_200_lower.startswith('<html>') and content[-7:].lower().startswith('</html>'):
+        return True
+
+    return False
+
+
+def initialize_rich_logging():
+    # Rich logging
+    logging.basicConfig(
+        level="INFO",
+        format="%(message)s",
+        datefmt="[%X]",
+        handlers=[RichHandler(rich_tracebacks=True)]
+    )
+
+    # Turn down 3rd party logging
+    logging.getLogger("httpx").setLevel(logging.WARNING)
+
+# Turn on rich logging if the environment variable is set
+if os.getenv('EDGAR_USE_RICH_LOGGING', '0') == '1':
+    initialize_rich_logging()
\ No newline at end of file
