@@ -1,11 +1,9 @@
-import asyncio
 import logging
 import os
 import re
 from dataclasses import dataclass
 from functools import lru_cache, cached_property
 from itertools import zip_longest
-from pathlib import Path
 from typing import List, Dict, Optional, Union, Tuple, Any
 
 import httpx
@@ -22,15 +20,16 @@ from rich.panel import Panel
 from rich.table import Table, Column
 from rich.text import Text
 
-from edgar._filings import Filing, Filings, FilingsState
+from edgar._filings import Filing, Filings, PagingState
 from edgar.company_reports import TenK, TenQ
-from edgar.core import (log, Result, display_size, listify, is_using_local_storage,
-                        filter_by_date, IntString, InvalidDateException, reverse_name, get_edgar_data_directory,
-                        datefmt)
+from edgar.core import (log, Result, display_size, listify,
+                        filter_by_date, IntString, InvalidDateException, reverse_name,
+                        parse_acceptance_datetime, datefmt)
+from edgar.storage import is_using_local_storage, get_edgar_data_directory
 from edgar.financials import Financials
-from edgar.httprequests import download_json, download_text, download_bulk_data
-from edgar.reference.tickers import get_company_tickers, get_icon_from_ticker, find_cik
+from edgar.httprequests import download_json, download_text
 from edgar.reference.forms import describe_form
+from edgar.reference.tickers import get_company_tickers, get_icon_from_ticker, find_cik, find_ticker
 from edgar.richtools import df_to_rich_table, repr_rich
 from edgar.search.datasearch import FastSearch, company_ticker_preprocess, company_ticker_score
 
@@ -53,6 +52,8 @@ __all__ = [
     'get_company_facts',
     'get_company_tickers',
     'get_entity_submissions',
+    'download_entity_submissions_from_sec',
+    'empty_company_filings',
     'parse_entity_submissions',
     'get_ticker_to_cik_lookup',
     'get_cik_lookup_data'
@@ -148,6 +149,7 @@ class EntityFiling(Filing):
                  form: str,
                  filing_date: str,
                  report_date: str,
+                 acceptance_datetime: str,
                  accession_no: str,
                  file_number: str,
                  items: str,
@@ -158,6 +160,7 @@ class EntityFiling(Filing):
                  is_inline_xbrl: bool):
         super().__init__(cik=cik, company=company, form=form, filing_date=filing_date, accession_no=accession_no)
         self.report_date = report_date
+        self.acceptance_datetime = acceptance_datetime
         self.file_number: str = file_number
         self.items: str = items
         self.size: int = size
@@ -217,7 +220,7 @@ class EntityFilings(Filings):
                  data: pa.Table,
                  cik: int,
                  company_name: str,
-                 original_state: FilingsState = None):
+                 original_state: PagingState = None):
         super().__init__(data, original_state=original_state)
         self.cik = cik
         self.company_name = company_name
@@ -236,6 +239,7 @@ class EntityFilings(Filings):
             form=self.data['form'][item].as_py(),
             filing_date=self.data['filing_date'][item].as_py(),
             report_date=self.data['reportDate'][item].as_py(),
+            acceptance_datetime=self.data['acceptanceDateTime'][item].as_py(),
             accession_no=self.data['accession_number'][item].as_py(),
             file_number=self.data['fileNumber'][item].as_py(),
             items=self.data['items'][item].as_py(),
@@ -255,7 +259,13 @@ class EntityFilings(Filings):
                ticker: Union[str, List[str]] = None,
                accession_number: Union[str, List[str]] = None, ):
         # The super filter returns Filings. We want CompanyFilings
-        res = super().filter(form, amendments, filing_date, date, accession_number)
+        res = super().filter(form=form,
+                             amendments=amendments,
+                             filing_date=filing_date,
+                             date=date,
+                             cik=cik,
+                             ticker=ticker,
+                             accession_number=accession_number)
         return CompanyFilings(data=res.data, cik=self.cik, company_name=self.company_name)
 
     def latest(self, n: int = 1):
@@ -308,7 +318,7 @@ class EntityFilings(Filings):
             log.warning("End of data .. use prev() \u2190 ")
             return None
         start_index, _ = self.data_pager._current_range
-        filings_state = FilingsState(page_start=start_index, num_filings=len(self))
+        filings_state = PagingState(page_start=start_index, num_records=len(self))
         return CompanyFilings(data_page,
                               cik=self.cik,
                               company_name=self.company_name,
@@ -324,7 +334,7 @@ class EntityFilings(Filings):
             log.warning(" No previous data .. use next() \u2192 ")
             return None
         start_index, _ = self.data_pager._current_range
-        filings_state = FilingsState(page_start=start_index, num_filings=len(self))
+        filings_state = PagingState(page_start=start_index, num_records=len(self))
         return CompanyFilings(data_page,
                               cik=self.cik,
                               company_name=self.company_name,
@@ -376,7 +386,7 @@ class EntityFilings(Filings):
         elements = [table]
 
         if self.data_pager.total_pages > 1:
-            total_filings = self._original_state.num_filings
+            total_filings = self._original_state.num_records
             current_count = len(current_page)
             start_num = start_idx + 1
             end_num = start_idx + current_count
@@ -593,6 +603,9 @@ class EntityData:
         combined_tables = pa.concat_tables(filing_tables)
         self.filings = CompanyFilings(combined_tables, cik=self.cik, company_name=self.name)
 
+    def get_empty_filings(self):
+        return empty_company_filings(self.cik, self.name)
+
     def get_filings(self,
                     *,
                     form: Union[str, List] = None,
@@ -783,8 +796,16 @@ class EntityData:
 
         # Primary entity identification section
         if self.is_company:
-            ticker_display = f" ({self.ticker_display})" if self.ticker_display else ""
-            entity_title = Text.assemble("🏢", (f"{self.display_name}{ticker_display}", "bold deep_sky_blue3"))
+            ticker = find_ticker(self.cik)
+            ticker = f"{ticker}" if ticker else ""
+
+            # The title of the panel
+            entity_title = Text.assemble("🏢 ",
+                                  (self.display_name, "bold green"),
+                                  " ",
+                                  (f"[{self.cik}] ", "dim"),
+                                  (ticker, "bold yellow")
+                                  )
         else:
             entity_title = Text.assemble("👤", (self.display_name, "bold green"))
 
@@ -806,9 +827,9 @@ class EntityData:
 
         # Detailed Information Table
         details = Table(box=box.SIMPLE, show_header=True, padding=(0, 1))
-        details.add_column("Category", style="bold grey70")
-        details.add_column("Industry", style="bold grey70")
-        details.add_column("Fiscal Year End", style="bold grey70")
+        details.add_column("Category")
+        details.add_column("Industry")
+        details.add_column("Fiscal Year End")
 
         details.add_row(
             self.category or "-",
@@ -830,8 +851,8 @@ class EntityData:
         # Trading Information
         if self.tickers and self.exchanges:
             trading_info = Table(box=box.SIMPLE, show_header=True, padding=(0, 1))
-            trading_info.add_column("Exchange", style="bold grey70")
-            trading_info.add_column("Symbol", style="bold deep_sky_blue3")
+            trading_info.add_column("Exchange")
+            trading_info.add_column("Symbol", style="bold yellow")
 
             for exchange, ticker in zip_longest(self.exchanges, self.tickers, fillvalue="-"):
                 trading_info.add_row(exchange, ticker)
@@ -865,13 +886,13 @@ class EntityData:
         contact_renderables = []
         if not self.business_address.empty:
             contact_renderables.append(Panel(
-                Text(str(self.business_address), style="grey85"),
+                Text(str(self.business_address)),
                 title="🏢 Business Address",
                 border_style="grey50"
             ))
         if not self.mailing_address.empty:
             contact_renderables.append(Panel(
-                Text(str(self.mailing_address), style="grey85"),
+                Text(str(self.mailing_address)),
                 title="📫 Mailing Address",
                 border_style="grey50"
             ))
@@ -885,8 +906,8 @@ class EntityData:
         # Former Names Table (if any exist)
         if self.former_names:
             former_names_table = Table(box=box.SIMPLE, show_header=False, padding=(0, 1))
-            former_names_table.add_column("Previous Company Names", style="bold grey70")
-            former_names_table.add_column("", style="grey85")  # Empty column for better spacing
+            former_names_table.add_column("Previous Company Names")
+            former_names_table.add_column("")  # Empty column for better spacing
 
             for former_name in self.former_names:
                 from_date = datefmt(former_name['from'], '%B %Y')
@@ -935,19 +956,11 @@ CompanyFilings = EntityFilings
 CompanyFacts = EntityFacts
 CompanyData = EntityData
 
-
-def extract_company_filings_table(filings_json: Dict[str, Any]) -> pa.Table:
-    """
-       Extract company filings from the json response
-       """
-    # Handle case of no data
-    if not filings_json['accessionNumber']:
-
-        schema = pa.schema([
+COMPANY_FILINGS_SCHEMA = schema = pa.schema([
             ('accession_number', pa.string()),
             ('filing_date', pa.date32()),
             ('reportDate', pa.string()),
-            ('acceptanceDateTime', pa.string()),
+            ('acceptanceDateTime', pa.timestamp('us')),  # Changed to timestamp
             ('act', pa.string()),
             ('form', pa.string()),
             ('fileNumber', pa.string()),
@@ -959,14 +972,33 @@ def extract_company_filings_table(filings_json: Dict[str, Any]) -> pa.Table:
             ('primaryDocDescription', pa.string())
         ])
 
-        filings_table = pa.Table.from_arrays([[] for _ in range(13)], schema=schema)
+def empty_company_filings_table():
+    """
+    Create an empty company filings table
+    """
+    return pa.Table.from_arrays([[] for _ in range(13)], schema=COMPANY_FILINGS_SCHEMA)
+
+def empty_company_filings(cik:IntString, company_name:str):
+    return CompanyFilings(empty_company_filings_table(), cik=cik, company_name=company_name)
+
+def extract_company_filings_table(filings_json: Dict[str, Any]) -> pa.Table:
+    """
+    Extract company filings from the json response
+    """
+    # Handle case of no data
+    if not filings_json['accessionNumber']:
+        filings_table = empty_company_filings_table()
     else:
+        # Convert acceptanceDateTime string to datetime
+        acceptance_datetimes = [
+            parse_acceptance_datetime(dt) for dt in filings_json['acceptanceDateTime']
+        ]
 
         fields = {
             'accession_number': filings_json['accessionNumber'],
             'filing_date': pc.cast(pc.strptime(pa.array(filings_json['filingDate']), '%Y-%m-%d', 'us'), pa.date32()),
             'reportDate': filings_json['reportDate'],
-            'acceptanceDateTime': filings_json['acceptanceDateTime'],
+            'acceptanceDateTime': acceptance_datetimes,  # Now passing datetime objects
             'act': filings_json['act'],
             'form': filings_json['form'],
             'fileNumber': filings_json['fileNumber'],
@@ -980,7 +1012,8 @@ def extract_company_filings_table(filings_json: Dict[str, Any]) -> pa.Table:
 
         # Create table using dictionary
         filings_table = pa.Table.from_arrays(
-            arrays=[pa.array(v) if k != 'filing_date' else v for k, v in fields.items()],
+            arrays=[pa.array(v) if k not in ['filing_date', 'acceptanceDateTime']
+                    else v for k, v in fields.items()],
             names=list(fields.keys())
         )
     return filings_table
@@ -1118,7 +1151,6 @@ Entity = get_entity
 @lru_cache(maxsize=32)
 def get_entity_submissions(cik: int) -> Optional[EntityData]:
     # Check the environment var EDGAR_USE_LOCAL_DATA
-    submissions_json: Optional[Dict[str, Any]] = None
     if is_using_local_storage():
         submissions_json = load_company_submissions_from_local(cik)
         if not submissions_json:
@@ -1150,22 +1182,24 @@ def parse_company_facts(fjson: Dict[str, object]):
     columns = ['namespace', 'fact', 'val', 'accn', 'start', 'end', 'fy', 'fp', 'form', 'filed', 'frame']
 
     # facts must be present
-    if 'facts' in fjson:
+    if 'facts' in fjson and fjson['facts']:
         for namespace, namespace_json in fjson['facts'].items():
             for fact, fact_json in namespace_json.items():
                 # Metadata about the facts
                 fact_meta_lst.append({'fact': fact,
-                                    'label': fact_json['label'],
-                                    'description': fact_json['description']})
+                                      'label': fact_json['label'],
+                                      'description': fact_json['description']})
 
                 for unit_key, unit_json in fact_json['units'].items():
                     unit_data = (pd.DataFrame(unit_json)
-                                .assign(namespace=namespace,
-                                        fact=fact,
-                                        label=fact_json['label'])
-                                .filter(columns)
-                                )
+                                 .assign(namespace=namespace,
+                                         fact=fact,
+                                         label=fact_json['label'])
+                                 .filter(columns)
+                                 )
                     unit_dfs.append(unit_data)
+    else:
+        return None
 
     # can't concatenate an empty list
     if len(unit_dfs) > 0:
@@ -1247,36 +1281,6 @@ def load_company_submissions_from_local(cik: int) -> Optional[Dict[str, Any]]:
     return json.loads(submissions_file.read_text())
 
 
-async def download_facts_async() -> Path:
-    """
-    Download company facts
-    """
-    log.info(f"Downloading Company facts to {get_edgar_data_directory()}/companyfacts")
-    return await download_bulk_data("https://www.sec.gov/Archives/edgar/daily-index/xbrl/companyfacts.zip")
-
-
-def download_facts() -> Path:
-    """
-    Download company facts
-    """
-    return asyncio.run(download_facts_async())
-
-
-async def download_submissions_async() -> Path:
-    """
-    Download company submissions
-    """
-    log.info(f"Downloading Company submissions to {get_edgar_data_directory()}/submissions")
-    return await download_bulk_data("https://www.sec.gov/Archives/edgar/daily-index/bulkdata/submissions.zip")
-
-
-def download_submissions() -> Path:
-    """
-    Download company facts
-    """
-    return asyncio.run(download_submissions_async())
-
-
 @dataclass(frozen=True)
 class Concept:
     taxonomy: str
