@@ -17,7 +17,7 @@ from rich.text import Text
 from edgar.core import log
 from edgar.files.html_documents import HtmlDocument, DocumentData
 from edgar.files.styles import StyleInfo, Width, parse_style, get_heading_level
-from edgar.files.tables import ProcessedTable, TableProcessor
+from edgar.files.tables import ProcessedTable, TableProcessor, ColumnOptimizer
 from edgar.richtools import repr_rich
 
 __all__ = ['SECHTMLParser', 'Document', 'DocumentNode']
@@ -201,7 +201,7 @@ class TextBlockNode(BaseNode):
 
 @dataclass
 class TableCell:
-    content: str
+    content: Union[str, BaseNode]
     colspan: int = 1
     rowspan: int = 1
     align: str = 'left'
@@ -266,11 +266,6 @@ class TableNode(BaseNode):
             return len(self._processed.data_rows[0])
         return 0
 
-    def force_process(self) -> None:
-        """Force table processing if needed"""
-        if self._processed_table is None:
-            self._processed_table = TableProcessor.process_table(self)
-
     def reset_processing(self) -> None:
         """Clear cached processed table"""
         self._processed_table = None
@@ -286,6 +281,10 @@ class TableNode(BaseNode):
         if not processed_table:
             return None
 
+        # Optimize the table
+        column_optimizer:ColumnOptimizer = ColumnOptimizer()
+        widths, processed_table = column_optimizer.optimize_columns(processed_table)
+
         table = Table(
             box=box.SIMPLE,
             border_style="blue",
@@ -532,7 +531,7 @@ class SECHTMLParser:
     def parse(self) -> Optional[Document]:
         body = self.root.find('body')
         if not body:
-            log.warn("No body tag found in HTML")
+            log.warning("No body tag found in HTML")
             return None
 
         nodes = self._parse_element(body)
@@ -1110,14 +1109,29 @@ class SECHTMLParser:
         def process_row(row: Tag) -> TableRow:
             """Process row preserving cell structure"""
             cells = []
-            for td in row.find_all(['td', 'th']):
-                cells.extend(process_cell(td))
+            # Make non-recursive in case of nested tables
+            for td in row.find_all(['td', 'th'], recursive=False):
+                # Check if cell contains a nested table
+                nested_table = td.find('table')
+                if nested_table:
+                    # Create a TableNode from the nested table using _process_table
+                    table_node = self._process_table(nested_table)
+                    if table_node:
+                        # Store the table node in the cell content
+                        cells.extend([TableCell(
+                            content=table_node,  # We'll need to handle this special content later
+                            colspan=int(td.get('colspan', '1')),
+                            align=td.get('align', 'left')
+                        )])
+
+                else:
+                    cells.extend(process_cell(td))
 
             return TableRow(cells=cells, is_header=row.find_parent('thead') is not None)
 
         # Process all rows
         rows = []
-        for tr in element.find_all('tr'):
+        for tr in element.find_all('tr', recursive=False):
             row = process_row(tr)
             if row.cells:
                 rows.append(row)
