@@ -1,5 +1,8 @@
 import gzip
+import logging
 import os
+import shutil
+import tarfile
 import time
 import zipfile
 from collections import deque
@@ -9,16 +12,18 @@ from pathlib import Path
 from threading import Lock
 from typing import Union, Optional
 
-import httpx
+from httpx import RequestError, Response, AsyncClient
 import orjson as json
 from stamina import retry
 from tqdm import tqdm
 
-from edgar.core import text_extensions, edgar_mode, get_edgar_data_directory
+from edgar.core import text_extensions, get_edgar_data_directory
+from edgar.httpclient import http_client, async_http_client
 
 __all__ = ["get_with_retry", "get_with_retry_async", "stream_with_retry", "post_with_retry", "post_with_retry_async",
            "download_file", "download_file_async", "download_json", "download_json_async", "stream_file",
-           "download_text", "download_text_between_tags", "download_bulk_data", "download_datafile"]
+           "download_text", "download_text_between_tags", "download_bulk_data", "download_datafile",
+           "throttle_requests"]
 
 attempts = 6
 retry_timeout = 40
@@ -167,7 +172,28 @@ def with_identity(func):
     return wrapper
 
 
-@retry(on=httpx.RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
+def async_with_identity(func):
+    @wraps(func)
+    def wrapper(client, url, identity=None, identity_callable=None, *args, **kwargs):
+        if identity is None:
+            if identity_callable is not None:
+                identity = identity_callable()
+            else:
+                identity = os.environ.get("EDGAR_IDENTITY")
+        if identity is None:
+            raise IdentityNotSetException("User-Agent identity is not set")
+
+        headers = kwargs.get("headers", {})
+        headers["User-Agent"] = identity
+        kwargs["headers"] = headers
+
+        return func(client, url, identity=identity, identity_callable=identity_callable, *args, **kwargs)
+
+    return wrapper
+
+
+
+@retry(on=RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
 @with_identity
 @throttle_requests(requests_per_second=max_requests_per_second)
 def get_with_retry(url, identity=None, identity_callable=None, **kwargs):
@@ -186,20 +212,20 @@ def get_with_retry(url, identity=None, identity_callable=None, **kwargs):
     Raises:
         TooManyRequestsError: If the response status code is 429 (Too Many Requests).
     """
-    with httpx.Client(timeout=edgar_mode.http_timeout) as client:
+    with http_client() as client:
         response = client.get(url, **kwargs)
         if response.status_code == 429:
             raise TooManyRequestsError(url)
         elif is_redirect(response):
-            return get_with_retry(response.headers["Location"], identity=identity, identity_callable=identity_callable,
-                                  **kwargs)
+            return get_with_retry(url=response.headers["Location"], identity=identity, identity_callable=identity_callable,
+                                 **kwargs)
         return response
 
 
-@retry(on=httpx.RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
-@with_identity
+@retry(on=RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
+@async_with_identity
 @throttle_requests(requests_per_second=max_requests_per_second)
-async def get_with_retry_async(url, identity=None, identity_callable=None, **kwargs):
+async def get_with_retry_async(client: AsyncClient, url, identity=None, identity_callable=None, **kwargs):
     """
     Sends an asynchronous GET request with retry functionality and identity handling.
 
@@ -215,17 +241,16 @@ async def get_with_retry_async(url, identity=None, identity_callable=None, **kwa
     Raises:
         TooManyRequestsError: If the response status code is 429 (Too Many Requests).
     """
-    async with httpx.AsyncClient(timeout=edgar_mode.http_timeout) as client:
-        response = await client.get(url, **kwargs)
-        if response.status_code == 429:
-            raise TooManyRequestsError(url)
-        elif is_redirect(response):
-            return await get_with_retry_async(response.headers["Location"], identity=identity,
-                                              identity_callable=identity_callable, **kwargs)
-        return response
+    response = await client.get(url, **kwargs)
+    if response.status_code == 429:
+        raise TooManyRequestsError(url)
+    elif is_redirect(response):
+        return await get_with_retry_async(client=client, url=response.headers["Location"], identity=identity,
+                                            identity_callable=identity_callable, **kwargs)
+    return response
 
 
-@retry(on=httpx.RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
+@retry(on=RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
 @with_identity
 @throttle_requests(requests_per_second=max_requests_per_second)
 def stream_with_retry(url, identity=None, identity_callable=None, **kwargs):
@@ -244,19 +269,20 @@ def stream_with_retry(url, identity=None, identity_callable=None, **kwargs):
     Raises:
         TooManyRequestsError: If the response status code is 429 (Too Many Requests).
     """
-    with httpx.Client(timeout=edgar_mode.http_timeout) as client:
+    with http_client() as client:
         with client.stream("GET", url, **kwargs) as response:
             if response.status_code == 429:
                 raise TooManyRequestsError(url)
             elif is_redirect(response):
-                yield stream_with_retry(response.headers["Location"],
+                response = stream_with_retry(response.headers["Location"],
                                         identity=identity,
                                         identity_callable=identity_callable, **kwargs)
+                yield from response
             else:
                 yield response
 
 
-@retry(on=httpx.RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
+@retry(on=RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
 @with_identity
 @throttle_requests(requests_per_second=max_requests_per_second)
 def post_with_retry(url, data=None, json=None, identity=None, identity_callable=None,
@@ -278,7 +304,7 @@ def post_with_retry(url, data=None, json=None, identity=None, identity_callable=
     Raises:
         TooManyRequestsError: If the response status code is 429 (Too Many Requests).
     """
-    with httpx.Client(timeout=edgar_mode.http_timeout) as client:
+    with http_client() as client:
         response = client.post(url, data=data, json=json, **kwargs)
         if response.status_code == 429:
             raise TooManyRequestsError(url)
@@ -288,10 +314,11 @@ def post_with_retry(url, data=None, json=None, identity=None, identity_callable=
         return response
 
 
-@retry(on=httpx.RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
-@with_identity
+@retry(on=RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
+@async_with_identity
 @throttle_requests(requests_per_second=max_requests_per_second)
-async def post_with_retry_async(url,
+async def post_with_retry_async(client: AsyncClient, 
+                                url,
                                 data=None,
                                 json=None,
                                 identity=None,
@@ -313,17 +340,16 @@ async def post_with_retry_async(url,
     Raises:
         TooManyRequestsError: If the response status code is 429 (Too Many Requests).
     """
-    async with httpx.AsyncClient(timeout=edgar_mode.http_timeout) as client:
-        response = await client.post(url, data=data, json=json, **kwargs)
-        if response.status_code == 429:
-            raise TooManyRequestsError(url)
-        elif is_redirect(response):
-            return await post_with_retry_async(response.headers["Location"], data=data, json=json, identity=identity,
-                                               identity_callable=identity_callable, **kwargs)
-        return response
+    response = await client.post(url, data=data, json=json, **kwargs)
+    if response.status_code == 429:
+        raise TooManyRequestsError(url)
+    elif is_redirect(response):
+        return await post_with_retry_async(client, response.headers["Location"], data=data, json=json, identity=identity,
+                                            identity_callable=identity_callable, **kwargs)
+    return response
 
 
-def inspect_response(response: httpx.Response):
+def inspect_response(response: Response):
     """
     Check if the response is successful and raise an exception if not.
     """
@@ -349,7 +375,6 @@ def save_or_return_content(content: Union[str, bytes], path: Optional[Union[str,
     Args:
         content (str or bytes): The content to save or return.
         path (str or Path, optional): The path where the content should be saved. If None, return the content.
-        as_text (bool): Whether the content is text or binary.
 
     Returns:
         str or bytes or None: The content if not saved, or None if saved.
@@ -414,12 +439,13 @@ def download_file(url: str, as_text: bool = None, path: Optional[Union[str, Path
         else:
             file_content = response.content
 
+    path = Path(path) if path else None
     if path and path.is_dir():
         path = path / os.path.basename(url)
     return save_or_return_content(file_content, path)
 
 
-async def download_file_async(url: str, as_text: bool = None, path: Optional[Union[str, Path]] = None) -> Union[
+async def download_file_async(client: AsyncClient, url: str, as_text: bool = None, path: Optional[Union[str, Path]] = None) -> Union[
     str, bytes, None]:
     """
     Download a file from a URL asynchronously.
@@ -437,7 +463,7 @@ async def download_file_async(url: str, as_text: bool = None, path: Optional[Uni
         # Set the default based on the file extension
         as_text = url.endswith(text_extensions)
 
-    response = await get_with_retry_async(url)
+    response = await get_with_retry_async(client, url)
     inspect_response(response)
 
     if as_text:
@@ -460,10 +486,14 @@ async def download_file_async(url: str, as_text: bool = None, path: Optional[Uni
 CHUNK_SIZE = 4 * 1024 * 1024  # 4MB
 
 
-@retry(on=httpx.RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
+@retry(on=RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
 @with_identity
 @throttle_requests(requests_per_second=max_requests_per_second)
-async def stream_file(url: str, as_text: bool = None, path: Optional[Union[str, Path]] = None, **kwargs) -> Union[
+async def stream_file(url: str,
+                      as_text: bool = None,
+                      path: Optional[Union[str, Path]] = None,
+                      client: Optional[AsyncClient]=None,
+                      **kwargs) -> Union[
     str, bytes, None]:
     """
     Download a file from a URL asynchronously with progress bar using httpx.
@@ -473,6 +503,7 @@ async def stream_file(url: str, as_text: bool = None, path: Optional[Union[str,
         as_text (bool, optional): Whether to download the file as text or binary.
             If None, the default is determined based on the file extension. Defaults to None.
         path (str or Path, optional): The path where the file should be saved.
+        client: The httpx.AsyncClient instance
 
     Returns:
         str or bytes: The content of the downloaded file, either as text or binary data.
@@ -481,8 +512,8 @@ async def stream_file(url: str, as_text: bool = None, path: Optional[Union[str,
         # Set the default based on the file extension
         as_text = url.endswith(text_extensions)
 
-    async with httpx.AsyncClient(timeout=edgar_mode.http_timeout, headers=kwargs['headers']) as client:
-        async with client.stream('GET', url) as response:
+    async with async_http_client(client) as async_client:
+        async with async_client.stream('GET', url) as response:
             inspect_response(response)
             total_size = int(response.headers.get('Content-Length', 0))
 
@@ -494,13 +525,16 @@ async def stream_file(url: str, as_text: bool = None, path: Optional[Union[str,
                 # Download as binary
                 content = b''
                 progress_bar = tqdm(
-                    total=total_size / (1024 * 1024),  # Convert to MB
+                    total=total_size / (1024 * 1024),
                     unit='MB',
                     unit_scale=True,
                     unit_divisor=1024,
+                    leave=False,  # Force horizontal display
+                    position=0,  # Lock the position
+                    dynamic_ncols=True,  # Adapt to terminal width
                     bar_format='{l_bar}{bar}| {n:.2f}/{total:.2f}MB [{elapsed}<{remaining}, {rate_fmt}]',
                     desc=f"Downloading {os.path.basename(url)}",
-                    ascii=False  # Use Unicode characters for a nicer looking bar
+                    ascii=False
                 )
                 downloaded = 0
                 async for chunk in response.aiter_bytes(chunk_size=CHUNK_SIZE):
@@ -540,7 +574,7 @@ def download_text(url: str) -> Optional[str]:
     return download_file(url, as_text=True)
 
 
-async def download_json_async(data_url: str) -> dict:
+async def download_json_async(client: AsyncClient, data_url: str) -> dict:
     """
     Download JSON data from a URL asynchronously.
 
@@ -550,7 +584,7 @@ async def download_json_async(data_url: str) -> dict:
     Returns:
         dict: The parsed JSON data.
     """
-    content = await download_file_async(data_url, as_text=True)
+    content = await download_file_async(client=client, url=data_url, as_text=True)
     return json.loads(content)
 
 
@@ -586,29 +620,100 @@ def download_text_between_tags(url: str, tag: str):
     return content
 
 
-async def download_bulk_data(data_url: str) -> Path:
+
+
+logger = logging.getLogger(__name__)
+
+
+@retry(on=RequestError, attempts=attempts, timeout=retry_timeout, wait_initial=wait_initial)
+async def download_bulk_data(url: str,
+                             data_directory: Path = get_edgar_data_directory(),
+                             client: Optional[AsyncClient] = None) -> Path:
     """
-    Download bulk data e.g. company facts, daily index, etc. from the SEC website
-    e.g. "https://www.sec.gov/Archives/edgar/daily-index/xbrl/companyfacts.zip"
+    Download and extract bulk data from zip or tar.gz archives
+
+    Args:
+        client: The httpx.AsyncClient instance
+        url: URL to download from (e.g. "https://www.sec.gov/Archives/edgar/daily-index/xbrl/companyfacts.zip")
+        data_directory: Base directory for downloads
+
+    Returns:
+        Path to the directory containing the extracted data
+
+    Raises:
+        ValueError: If the URL or filename is invalid
+        IOError: If there are file system operation failures
+        zipfile.BadZipFile: If the downloaded zip file is corrupted
+        tarfile.TarError: If the downloaded tar.gz file is corrupted
     """
-    filename = os.path.basename(data_url)
+    if not url:
+        raise ValueError("Data URL cannot be empty")
+
+    filename = os.path.basename(url)
+    if not filename:
+        raise ValueError("Invalid URL - cannot extract filename")
+
     local_dir = filename.split('.')[0]
-    download_path = get_edgar_data_directory() / local_dir
+    download_path = data_directory / local_dir
     download_filename = download_path / filename
 
-    # Create the directory if it doesn't exist
-    if not download_path.exists():
-        download_path.mkdir()
+    try:
+        # Create the directory with parents=True and exist_ok=True to avoid race conditions
+        download_path.mkdir(parents=True, exist_ok=True)
+
+        # Download the file
+        try:
+            await stream_file(url, client=client, path=download_path)
+        except Exception as e:
+            raise IOError(f"Failed to download file: {e}")
+
+        # Extract based on file extension
+        try:
+            if filename.endswith(".zip"):
+                with zipfile.ZipFile(download_filename, 'r') as z:
+                    z.extractall(download_path)
+            elif any(filename.endswith(ext) for ext in (".tar.gz", ".tgz")):
+                with tarfile.open(download_filename, 'r:gz') as tar:
+                    # Security check for tar files to prevent path traversal
+                    def is_within_directory(directory: Path, target: Path) -> bool:
+                        try:
+                            return os.path.commonpath([directory, target]) == str(directory)
+                        except ValueError:
+                            return False
+
+                    def safe_extract(tar: tarfile.TarFile, path: str) -> None:
+                        for member in tar.getmembers():
+                            member_path = os.path.join(path, member.name)
+                            if not is_within_directory(Path(path), Path(member_path)):
+                                raise ValueError(f"Attempted path traversal in tar file: {member.name}")
+                        tar.extractall(path)
+
+                    safe_extract(tar, str(download_path))
+            else:
+                raise ValueError(f"Unsupported file format: {filename}")
+
+        except (zipfile.BadZipFile, tarfile.TarError) as e:
+            raise type(e)(f"Failed to extract archive {filename}: {e}")
+
+        finally:
+            # Always try to clean up the archive file, but don't fail if we can't
+            try:
+                if download_filename.exists():
+                    download_filename.unlink()
+            except Exception as e:
+                logger.warning(f"Failed to delete archive file {download_filename}: {e}")
+
+        return download_path
+
+    except Exception:
+        # Clean up the download directory in case of any errors
+        try:
+            if download_path.exists():
+                shutil.rmtree(download_path)
+        except Exception as cleanup_error:
+            logger.error(f"Failed to clean up after error: {cleanup_error}")
+        raise
 
-    if filename.endswith(".zip"):
-        # Now stream the file to the data directory
-        await stream_file(data_url, path=download_path)
-        # Unzip the file to the data directory / file
-        with zipfile.ZipFile(download_filename, 'r') as z:
-            z.extractall(download_path)
-        # Delete the zip file
-        download_filename.unlink()
-    return download_path
 
 
 def download_datafile(data_url: str, local_directory:Path=None) -> Path:
