@@ -24,7 +24,7 @@ from rich.tree import Tree
 
 from edgar.datatools import na_value
 from edgar.richtools import repr_rich, colorize_words
-from edgar.attachments import Attachments
+from edgar.attachments import Attachments, Attachment
 from edgar.core import log, split_camel_case, run_async_or_sync
 from edgar.httprequests import download_file_async
 from edgar.xbrl.calculations import CalculationLinkbase
@@ -37,6 +37,7 @@ from edgar.xbrl.presentation import (XBRLPresentation, PresentationElement, get_
     get_members_for_axis)
 from pathlib import Path
 
+
 __all__ = ['XBRLAttachments', 'XBRLInstance', 'LineItem', 'StatementDefinition', 'XBRLData',
            'Statements', 'Statement']
 
@@ -70,8 +71,10 @@ class XBRLAttachments:
         self._documents = dict()
         if attachments.data_files:
             for attachment in attachments.data_files:
-                if attachment.document_type in ['XML', 'EX-101.INS']:
-                    self._documents['instance'] = attachment
+                if attachment.document_type in ["XML", 'EX-101.INS'] and attachment.extension.endswith(('.xml', '.XML')):
+                    content = attachment.content
+                    if '<xbrl' in content[:2000]:
+                        self._documents['instance'] = attachment
                 elif attachment.document_type == 'EX-101.SCH':
                     self._documents['schema'] = attachment
                 elif attachment.document_type == 'EX-101.DEF':
@@ -119,25 +122,26 @@ class XBRLAttachments:
         """
         Load the XBRL documents asynchronously and parse them.
         """
+
         parsers = {
-            'definition': parse_definition_linkbase,
-            'label': parse_label_linkbase,
-            'calculation': CalculationLinkbase.parse,
-            'presentation': lambda x: x,
-            'instance': lambda x: x,
-            'schema': lambda x: x
-        }
+                'definition': parse_definition_linkbase,
+                'label': parse_label_linkbase,
+                'calculation': CalculationLinkbase.parse,
+                'presentation': lambda x: x,
+                'instance': lambda x: x,
+                'schema': lambda x: x
+            }
         parsed_files = {}
 
         # Download all files concurrently
-        download_tasks = []
+        load_tasks = []
         for doc_type in ['instance', 'schema', 'label', 'calculation', 'presentation']:
             attachment = self.get(doc_type)
             if attachment:
-                download_tasks.append(XBRLAttachments.download_and_parse(doc_type, parsers[doc_type], attachment.url))
+                load_tasks.append(XBRLAttachments.parse_content(doc_type, parsers[doc_type], attachment))
 
         # Wait for all downloads to complete
-        results = await asyncio.gather(*download_tasks)
+        results = await asyncio.gather(*load_tasks)
         for result in results:
             parsed_files.update(result)
 
@@ -155,9 +159,38 @@ class XBRLAttachments:
                 parsed_files.get('label', {}),
                 parsed_files.get('calculation'))
 
+    def load_content(self):
+        """
+        Load the XBRL documents and parse them.
+        """
+        if self.empty:
+            return None
+        elif self.instance_only:
+            return XBRLInstance.parse(self._documents['instance'].download())
+        else:
+            instance_xml, presentation_xml, labels, calculations = asyncio.run(self.load())
+            return XBRLData.parse(instance_xml=instance_xml, presentation_xml=presentation_xml, labels=labels,
+                                  calculations=calculations)
+
     @staticmethod
-    async def download_and_parse(doc_type: str, parser, url: str):
-        content = await download_file_async(url)
+    async def parse_content(doc_type: str, parser, attachment:Attachment):
+        """
+        Parse the content of an attachment asynchronously.
+        doc_type: str: The type of document being parsed.
+        parser: Callable: The parser function to use.
+        attachment: Attachment: The attachment to parse.
+        """
+        return {doc_type: parser(attachment.content)}
+
+    @staticmethod
+    async def download_and_parse(client, doc_type: str, parser, url: str):
+        """
+        Download and parse a document asynchronously.
+        doc_type: str: The type of document being parsed.
+        parser: Callable: The parser function to use.
+        url: str: The URL of the document to download.
+        """
+        content = await download_file_async(client, url)
         return {doc_type: parser(content)}
 
     @staticmethod
@@ -379,7 +412,7 @@ class StatementDefinition():
                labels: Dict,
                xbrl_data: 'XBRLData') -> 'StatementDefinition':
 
-        if not role or not presentation_element or not labels or not xbrl_data:
+        if not role or not presentation_element or not xbrl_data:
             raise ValueError("All parameters are required")
 
         # Factory method to create a StatementDefinition instance
@@ -1402,7 +1435,9 @@ class XBRLData():
         parsed_documents = await xbrl_documents.load()
         if parsed_documents:
             instance_xml, presentation_xml, labels, calculations = parsed_documents
-            return cls.parse(instance_xml=instance_xml, presentation_xml=presentation_xml, labels=labels,
+            return cls.parse(instance_xml=instance_xml,
+                             presentation_xml=presentation_xml,
+                             labels=labels,
                              calculations=calculations)
 
     @classmethod
